@@ -124,7 +124,8 @@ done:
 	return initial_vector;
 }
 
-static CK_RV encrypt_data(CK_BYTE *key, CK_ULONG keylen, const CK_BYTE *iv,
+static CK_RV encrypt_data(STDLL_TokData_t *tokdata, CK_BYTE *key,
+			  CK_ULONG keylen, const CK_BYTE *iv,
 			  CK_BYTE *clear, CK_ULONG clear_len,
 			  CK_BYTE *cipher, CK_ULONG *p_cipher_len)
 {
@@ -152,8 +153,8 @@ static CK_RV encrypt_data(CK_BYTE *key, CK_ULONG keylen, const CK_BYTE *iv,
 		TRACE_ERROR("%s\n", ock_err(ERR_MECHANISM_INVALID));
 		return ERR_MECHANISM_INVALID;
 	}
-	rc = object_create_skel(key_tmpl, 3, MODE_CREATE, CKO_SECRET_KEY,
-				keyType, &keyobj);
+	rc = object_create_skel(tokdata, key_tmpl, 3, MODE_CREATE,
+				CKO_SECRET_KEY, keyType, &keyobj);
 	if (rc != CKR_OK) {
 		TRACE_DEVEL("object_create_skel failed.\n");
 		return rc;
@@ -167,12 +168,12 @@ static CK_RV encrypt_data(CK_BYTE *key, CK_ULONG keylen, const CK_BYTE *iv,
 
 	switch (token_specific.data_store.encryption_algorithm) {
 	case CKM_DES3_CBC:
-		rc = ckm_des3_cbc_encrypt(clear, clear_len,
+		rc = ckm_des3_cbc_encrypt(tokdata, clear, clear_len,
 					  cipher, p_cipher_len,
 					  initial_vector, keyobj);
 		break;
 	case CKM_AES_CBC:
-		rc = ckm_aes_cbc_encrypt(clear, clear_len,
+		rc = ckm_aes_cbc_encrypt(tokdata, clear, clear_len,
 		                         cipher, p_cipher_len,
 					 initial_vector, keyobj);
 		break;
@@ -192,7 +193,8 @@ static CK_RV encrypt_data(CK_BYTE *key, CK_ULONG keylen, const CK_BYTE *iv,
 #endif
 }
 
-static CK_RV encrypt_data_with_clear_key(CK_BYTE *key, CK_ULONG keylen,
+static CK_RV encrypt_data_with_clear_key(STDLL_TokData_t * tokdata,
+					 CK_BYTE *key, CK_ULONG keylen,
 					 const CK_BYTE *iv,
 					 CK_BYTE *clear, CK_ULONG clear_len,
 					 CK_BYTE *cipher, CK_ULONG *p_cipher_len)
@@ -205,7 +207,7 @@ static CK_RV encrypt_data_with_clear_key(CK_BYTE *key, CK_ULONG keylen,
 	 * clear key.
 	 */
 	if (token_specific.token_keysize == 0) {
-		return encrypt_data(key, keylen, iv, clear, clear_len,
+		return encrypt_data(tokdata, key, keylen, iv, clear, clear_len,
 				    cipher, p_cipher_len);
 	}
 
@@ -238,7 +240,8 @@ static CK_RV encrypt_data_with_clear_key(CK_BYTE *key, CK_ULONG keylen,
 #endif
 }
 
-static CK_RV decrypt_data(CK_BYTE *key, CK_ULONG keylen, const CK_BYTE *iv,
+static CK_RV decrypt_data(STDLL_TokData_t *tokdata,
+			  CK_BYTE *key, CK_ULONG keylen, const CK_BYTE *iv,
 			  CK_BYTE *cipher, CK_ULONG cipher_len,
 			  CK_BYTE *clear, CK_ULONG *p_clear_len)
 {
@@ -266,8 +269,8 @@ static CK_RV decrypt_data(CK_BYTE *key, CK_ULONG keylen, const CK_BYTE *iv,
 		TRACE_ERROR("%s\n", ock_err(ERR_MECHANISM_INVALID));
 		return ERR_MECHANISM_INVALID;
 	}
-	rc = object_create_skel(key_tmpl, 3, MODE_CREATE, CKO_SECRET_KEY,
-				keyType, &keyobj);
+	rc = object_create_skel(tokdata, key_tmpl, 3, MODE_CREATE,
+				CKO_SECRET_KEY, keyType, &keyobj);
 	if (rc != CKR_OK) {
 		TRACE_DEVEL("object_create_skel failed.\n");
 		return rc;
@@ -281,12 +284,12 @@ static CK_RV decrypt_data(CK_BYTE *key, CK_ULONG keylen, const CK_BYTE *iv,
 
 	switch (token_specific.data_store.encryption_algorithm) {
 	case CKM_DES3_CBC:
-		rc = ckm_des3_cbc_decrypt(cipher, cipher_len,
+		rc = ckm_des3_cbc_decrypt(tokdata, cipher, cipher_len,
 					  clear, p_clear_len,
 					  initial_vector, keyobj);
 		break;
 	case CKM_AES_CBC:
-		rc = ckm_aes_cbc_decrypt(cipher, cipher_len,
+		rc = ckm_aes_cbc_decrypt(tokdata, cipher, cipher_len,
 					 clear, p_clear_len,
 					 initial_vector, keyobj);
 		break;
@@ -306,7 +309,8 @@ static CK_RV decrypt_data(CK_BYTE *key, CK_ULONG keylen, const CK_BYTE *iv,
 #endif
 }
 
-static CK_RV decrypt_data_with_clear_key(CK_BYTE *key, CK_ULONG keylen,
+static CK_RV decrypt_data_with_clear_key(STDLL_TokData_t *tokdata,
+					 CK_BYTE *key, CK_ULONG keylen,
 					 const CK_BYTE *iv,
 					 CK_BYTE *cipher, CK_ULONG cipher_len,
 					 CK_BYTE *clear, CK_ULONG *p_clear_len)
@@ -319,8 +323,8 @@ static CK_RV decrypt_data_with_clear_key(CK_BYTE *key, CK_ULONG keylen,
 	 * clear key.
 	 */
 	if (token_specific.token_keysize == 0) {
-		return decrypt_data(key, keylen, iv, cipher, cipher_len,
-				    clear, p_clear_len);
+		return decrypt_data(tokdata, key, keylen, iv, cipher,
+				    cipher_len, clear, p_clear_len);
 	}
 
 	/* Fall back to a software alternative if key is secure. */
@@ -384,21 +388,20 @@ error:
 //
 //
 CK_RV
-load_token_data(CK_SLOT_ID slot_id)
+load_token_data(STDLL_TokData_t *tokdata, CK_SLOT_ID slot_id)
 {
 	FILE *fp = NULL;
 	CK_BYTE fname[PATH_MAX];
 	TOKEN_DATA td;
 	CK_RV rc;
-	char pk_dir_buf[PATH_MAX];
 
-	rc = XProcLock();
+	rc = XProcLock(tokdata);
 	if (rc != CKR_OK) {
 		TRACE_ERROR("Failed to get Process Lock.\n");
 		goto out_nolock;
 	}
 
-	sprintf(fname, "%s/%s", get_pk_dir(pk_dir_buf), PK_LITE_NV);
+	sprintf(fname, "%s/%s", tokdata->data_store, PK_LITE_NV);
 	fp = fopen((char *)fname, "r");
 	if (!fp) {
 		/* Better error checking added */
@@ -406,9 +409,9 @@ load_token_data(CK_SLOT_ID slot_id)
 			/* init_token_data may call save_token_data, which
 			 * grabs the lock, so we must release it around this
 			 * call */
-			XProcUnLock();
-			init_token_data(slot_id);
-			rc = XProcLock();
+			XProcUnLock(tokdata);
+			init_token_data(tokdata, slot_id);
+			rc = XProcLock(tokdata);
 			if (rc != CKR_OK) {
 				TRACE_ERROR("Failed to get Process Lock.\n");
 				goto out_nolock;
@@ -437,11 +440,11 @@ load_token_data(CK_SLOT_ID slot_id)
 		rc = CKR_FUNCTION_FAILED;
 		goto out_unlock;
 	}
-	memcpy(nv_token_data, &td, sizeof(TOKEN_DATA));
+	memcpy(tokdata->nv_token_data, &td, sizeof(TOKEN_DATA));
 
 	/* Load token-specific data */
 	if (token_specific.t_load_token_data) {
-		rc = token_specific.t_load_token_data(slot_id, fp);
+		rc = token_specific.t_load_token_data(tokdata, slot_id, fp);
 		if (rc)
 			goto out_unlock;
 	}
@@ -449,7 +452,7 @@ load_token_data(CK_SLOT_ID slot_id)
 	rc = CKR_OK;
 
 out_unlock:
-	XProcUnLock();
+	XProcUnLock(tokdata);
 
 out_nolock:
 	if (fp)
@@ -459,34 +462,30 @@ out_nolock:
 
 //
 //
-CK_RV save_token_data(CK_SLOT_ID slot_id)
+CK_RV save_token_data(STDLL_TokData_t *tokdata, CK_SLOT_ID slot_id)
 {
 	FILE *fp = NULL;
 	TOKEN_DATA td;
 	CK_RV rc;
 	CK_BYTE fname[PATH_MAX];
-	char pk_dir_buf[PATH_MAX];
 
-	rc = XProcLock();
+	rc = XProcLock(tokdata);
 	if (rc != CKR_OK) {
 		TRACE_ERROR("Failed to get Process Lock.\n");
 		goto out_nolock;
 	}
 
-	sprintf(fname, "%s/%s", get_pk_dir(pk_dir_buf), PK_LITE_NV);
-	fp = fopen((char *)fname, "r+");
+	sprintf(fname, "%s/%s", tokdata->data_store, PK_LITE_NV);
+	fp = fopen((char *)fname, "w");
 	if (!fp) {
-		fp = fopen((char *)fname, "w");
-		if (!fp) {
-			TRACE_ERROR("fopen(%s): %s\n", fname, strerror(errno));
-			rc = CKR_FUNCTION_FAILED;
-			goto done;
-		}
+		TRACE_ERROR("fopen(%s): %s\n", fname, strerror(errno));
+		rc = CKR_FUNCTION_FAILED;
+		goto done;
 	}
 	set_perm(fileno(fp));
 
 	/* Write generic token data */
-	memcpy(&td, nv_token_data, sizeof(TOKEN_DATA));
+	memcpy(&td, tokdata->nv_token_data, sizeof(TOKEN_DATA));
 	if (!fwrite(&td, sizeof(TOKEN_DATA), 1, fp)) {
 		rc = CKR_FUNCTION_FAILED;
 		goto done;
@@ -494,7 +493,7 @@ CK_RV save_token_data(CK_SLOT_ID slot_id)
 
 	/* Write token-specific data */
 	if (token_specific.t_save_token_data) {
-		rc = token_specific.t_save_token_data(slot_id, fp);
+		rc = token_specific.t_save_token_data(tokdata, slot_id, fp);
 		if (rc)
 			goto done;
 	}
@@ -502,7 +501,7 @@ CK_RV save_token_data(CK_SLOT_ID slot_id)
 	rc = CKR_OK;
 
 done:
-	XProcUnLock();
+	XProcUnLock(tokdata);
 
 out_nolock:
 	if (fp)
@@ -512,37 +511,33 @@ out_nolock:
 
 //
 //
-CK_RV save_token_object(OBJECT * obj)
+CK_RV save_token_object(STDLL_TokData_t *tokdata, OBJECT * obj)
 {
 	FILE *fp = NULL;
 	CK_BYTE line[100];
 	CK_RV rc;
 	CK_BYTE fname[PATH_MAX];
-	char pk_dir_buf[PATH_MAX];
 
 	if (object_is_private(obj) == TRUE)
-		rc = save_private_token_object(obj);
+		rc = save_private_token_object(tokdata, obj);
 	else
-		rc = save_public_token_object(obj);
+		rc = save_public_token_object(tokdata, obj);
 
 	if (rc != CKR_OK)
 		return rc;
 
 	// update the index file if it exists
-	sprintf(fname, "%s/%s/%s", get_pk_dir(pk_dir_buf), PK_LITE_OBJ_DIR,
+	sprintf(fname, "%s/%s/%s", tokdata->data_store, PK_LITE_OBJ_DIR,
 		PK_LITE_OBJ_IDX);
 	fp = fopen((char *)fname, "r");
 	if (fp) {
 		set_perm(fileno(fp));
-		while (!feof(fp)) {
-			(void)fgets((char *)line, 50, fp);
-			if (!feof(fp)) {
-				line[strlen(line) - 1] = 0;
-				if (strcmp(line, obj->name) == 0) {
-					fclose(fp);
-					// object is already in the list
-					return CKR_OK;
-				}
+		while (fgets((char *)line, 50, fp)) {
+			line[strlen(line) - 1] = 0;
+			if (strcmp(line, obj->name) == 0) {
+				fclose(fp);
+				// object is already in the list
+				return CKR_OK;
 			}
 		}
 		fclose(fp);
@@ -566,7 +561,7 @@ CK_RV save_token_object(OBJECT * obj)
 // this is the same as the old version.  public token objects are stored in the
 // clear
 //
-CK_RV save_public_token_object(OBJECT * obj)
+CK_RV save_public_token_object(STDLL_TokData_t *tokdata, OBJECT * obj)
 {
 	FILE *fp = NULL;
 	CK_BYTE *clear = NULL;
@@ -575,14 +570,13 @@ CK_RV save_public_token_object(OBJECT * obj)
 	CK_BBOOL flag = FALSE;
 	CK_RV rc;
 	CK_ULONG_32 total_len;
-	char pk_dir_buf[PATH_MAX];
 
 	rc = object_flatten(obj, &clear, &clear_len);
 	if (rc != CKR_OK) {
 		goto error;
 	}
 
-	sprintf(fname, "%s/%s/", get_pk_dir(pk_dir_buf), PK_LITE_OBJ_DIR);
+	sprintf(fname, "%s/%s/", tokdata->data_store, PK_LITE_OBJ_DIR);
 	strncat((char *)fname, (char *)obj->name, 8);
 	fp = fopen((char *)fname, "w");
 	if (!fp) {
@@ -614,7 +608,7 @@ error:
 
 //
 //
-CK_RV save_private_token_object(OBJECT * obj)
+CK_RV save_private_token_object(STDLL_TokData_t *tokdata, OBJECT * obj)
 {
 	FILE *fp = NULL;
 	CK_BYTE *obj_data = NULL;
@@ -632,7 +626,6 @@ CK_RV save_private_token_object(OBJECT * obj)
 	CK_RV rc;
 	CK_ULONG_32 obj_data_len_32;
 	CK_ULONG_32 total_len;
-	char pk_dir_buf[PATH_MAX];
 
 	rc = object_flatten(obj, &obj_data, &obj_data_len);
 	obj_data_len_32 = obj_data_len;
@@ -648,7 +641,7 @@ CK_RV save_private_token_object(OBJECT * obj)
 	//       SHA of (object data)             |
 	//    ---- end encrypted part          <--+
 	//
-	compute_sha1(obj_data, obj_data_len, hash_sha);
+	compute_sha1(tokdata, obj_data, obj_data_len, hash_sha);
 
 	// encrypt the sensitive object data.  need to be careful.
 	// if I use the normal high-level encryption routines I'll need to
@@ -668,7 +661,7 @@ CK_RV save_private_token_object(OBJECT * obj)
 	key = malloc(key_len);
 	if (!key)
 		goto oom_error;
-	memcpy(key, master_key, key_len);
+	memcpy(key, tokdata->master_key, key_len);
 
 
 	clear_len = sizeof(CK_ULONG_32) + obj_data_len_32 + SHA1_HASH_SIZE;
@@ -690,14 +683,14 @@ CK_RV save_private_token_object(OBJECT * obj)
 	add_pkcs_padding(clear + clear_len, block_size, clear_len,
 			 padded_len);
 
-	rc = encrypt_data_with_clear_key(key, key_len,
+	rc = encrypt_data_with_clear_key(tokdata, key, key_len,
 			  token_specific.data_store.obj_initial_vector,
 			  clear, padded_len, cipher, &cipher_len);
 	if (rc != CKR_OK) {
 		goto error;
 	}
 
-	sprintf(fname, "%s/%s/", get_pk_dir(pk_dir_buf), PK_LITE_OBJ_DIR);
+	sprintf(fname, "%s/%s/", tokdata->data_store, PK_LITE_OBJ_DIR);
 	strncat((char *)fname, (char *)obj->name, 8);
 	fp = fopen((char *)fname, "w");
 	if (!fp) {
@@ -743,7 +736,7 @@ error:
 
 //
 //
-CK_RV load_public_token_objects(void)
+CK_RV load_public_token_objects(STDLL_TokData_t *tokdata)
 {
 	FILE *fp1 = NULL, *fp2 = NULL;
 	CK_BYTE *buf = NULL;
@@ -751,65 +744,70 @@ CK_RV load_public_token_objects(void)
 	CK_BBOOL priv;
 	CK_ULONG_32 size;
 	size_t read_size;
-	char pk_dir_buf[PATH_MAX];
 
-	sprintf(iname, "%s/%s/%s", get_pk_dir(pk_dir_buf), PK_LITE_OBJ_DIR,
+	sprintf(iname, "%s/%s/%s", tokdata->data_store, PK_LITE_OBJ_DIR,
 		PK_LITE_OBJ_IDX);
 
 	fp1 = fopen((char *)iname, "r");
 	if (!fp1)
 		return CKR_OK;	// no token objects
 
-	while (!feof(fp1)) {
-		(void)fgets((char *)tmp, 50, fp1);
-		if (!feof(fp1)) {
-			tmp[strlen((char *)tmp) - 1] = 0;
+	while (fgets((char *)tmp, 50, fp1)) {
+		tmp[strlen((char *)tmp) - 1] = 0;
 
-			sprintf((char *)fname, "%s/%s/", get_pk_dir(pk_dir_buf),
-				PK_LITE_OBJ_DIR);
-			strcat((char *)fname, (char *)tmp);
+		sprintf((char *)fname, "%s/%s/", tokdata->data_store,
+			PK_LITE_OBJ_DIR);
+		strcat((char *)fname, (char *)tmp);
 
-			fp2 = fopen((char *)fname, "r");
-			if (!fp2)
-				continue;
+		fp2 = fopen((char *)fname, "r");
+		if (!fp2)
+			continue;
 
-			fread(&size, sizeof(CK_ULONG_32), 1, fp2);
-			fread(&priv, sizeof(CK_BBOOL), 1, fp2);
-			if (priv == TRUE) {
-				fclose(fp2);
-				continue;
-			}
-			// size--;
-			size = size - sizeof(CK_ULONG_32) - sizeof(CK_BBOOL);
-			buf = (CK_BYTE *) malloc(size);
-			if (!buf) {
-				fclose(fp2);
-				OCK_SYSLOG(LOG_ERR,
-					   "Cannot malloc %u bytes to read in "
-					   "token object %s (ignoring it)",
-					   size, fname);
-				continue;
-			}
-
-			read_size = fread(buf, 1, size, fp2);
-			if (read_size != size) {
-				fclose(fp2);
-				free(buf);
-				OCK_SYSLOG(LOG_ERR,
-					   "Cannot read token object %s "
-					   "(ignoring it)", fname);
-				continue;
-			}
-			// ... grab object mutex here.
-			if (object_mgr_restore_obj_withSize(buf, NULL, size) !=
-							    CKR_OK) {
-				OCK_SYSLOG(LOG_ERR,
-					   "Cannot restore token object %s "
-					   "(ignoring it)", fname);
-			}
-			free(buf);
+		if (!fread(&size, sizeof(CK_ULONG_32), 1, fp2)) {
 			fclose(fp2);
+			OCK_SYSLOG(LOG_ERR, "Cannot read size\n");
+			continue;
 		}
+		if (!fread(&priv, sizeof(CK_BBOOL), 1, fp2)) {
+			fclose(fp2);
+			OCK_SYSLOG(LOG_ERR, "Cannot read boolean\n");
+			continue;
+		}
+		if (priv == TRUE) {
+			fclose(fp2);
+			continue;
+		}
+		// size--;
+		size = size - sizeof(CK_ULONG_32) - sizeof(CK_BBOOL);
+		buf = (CK_BYTE *) malloc(size);
+		if (!buf) {
+			fclose(fp2);
+			OCK_SYSLOG(LOG_ERR,
+				   "Cannot malloc %u bytes to read in "
+				   "token object %s (ignoring it)",
+				   size, fname);
+			continue;
+		}
+
+		read_size = fread(buf, 1, size, fp2);
+		if (read_size != size) {
+			fclose(fp2);
+			free(buf);
+			OCK_SYSLOG(LOG_ERR,
+				   "Cannot read token object %s "
+				   "(ignoring it)", fname);
+			continue;
+		}
+		// ... grab object mutex here.
+		if (object_mgr_restore_obj_withSize(tokdata, buf,
+						    NULL, size) !=
+						    CKR_OK) {
+			OCK_SYSLOG(LOG_ERR,
+				   "Cannot restore token object %s "
+				   "(ignoring it)", fname);
+		}
+		free(buf);
+		fclose(fp2);
 	}
 	fclose(fp1);
 
@@ -818,7 +816,7 @@ CK_RV load_public_token_objects(void)
 
 //
 //
-CK_RV load_private_token_objects(void)
+CK_RV load_private_token_objects(STDLL_TokData_t *tokdata)
 {
 	FILE *fp1 = NULL, *fp2 = NULL;
 	CK_BYTE *buf = NULL;
@@ -827,65 +825,70 @@ CK_RV load_private_token_objects(void)
 	CK_ULONG_32 size;
 	CK_RV rc;
 	size_t read_size;
-	char pk_dir_buf[PATH_MAX];
 
-	sprintf(iname, "%s/%s/%s", get_pk_dir(pk_dir_buf), PK_LITE_OBJ_DIR,
+	sprintf(iname, "%s/%s/%s", tokdata->data_store, PK_LITE_OBJ_DIR,
 		PK_LITE_OBJ_IDX);
 
 	fp1 = fopen((char *)iname, "r");
 	if (!fp1)
 		return CKR_OK;	// no token objects
 
-	while (!feof(fp1)) {
-		(void)fgets((char *)tmp, 50, fp1);
-		if (!feof(fp1)) {
-			tmp[strlen((char *)tmp) - 1] = 0;
+	while (fgets((char *)tmp, 50, fp1)) {
+		tmp[strlen((char *)tmp) - 1] = 0;
 
-			sprintf((char *)fname, "%s/%s/", get_pk_dir(pk_dir_buf),
-				PK_LITE_OBJ_DIR);
-			strcat((char *)fname, (char *)tmp);
+		sprintf((char *)fname, "%s/%s/", tokdata->data_store,
+			PK_LITE_OBJ_DIR);
+		strcat((char *)fname, (char *)tmp);
 
-			fp2 = fopen((char *)fname, "r");
-			if (!fp2)
-				continue;
+		fp2 = fopen((char *)fname, "r");
+		if (!fp2)
+			continue;
 
-			fread(&size, sizeof(CK_ULONG_32), 1, fp2);
-			fread(&priv, sizeof(CK_BBOOL), 1, fp2);
-			if (priv == FALSE) {
-				fclose(fp2);
-				continue;
-			}
-			//size--;
-			size = size - sizeof(CK_ULONG_32) - sizeof(CK_BBOOL);
-			buf = (CK_BYTE *) malloc(size);
-			if (!buf) {
-				fclose(fp2);
-				OCK_SYSLOG(LOG_ERR,
-					   "Cannot malloc %u bytes to read in "
-					   "token object %s (ignoring it)",
-					   size, fname);
-				continue;
-			}
-
-			read_size = fread((char *)buf, 1, size, fp2);
-			if (read_size != size) {
-				free(buf);
-				fclose(fp2);
-				OCK_SYSLOG(LOG_ERR,
-					   "Cannot read token object %s "
-					   "(ignoring it)", fname);
-				continue;
-			}
-			// Grab object list  mutex
-			MY_LockMutex(&obj_list_mutex);
-			rc = restore_private_token_object(buf, size, NULL);
-			MY_UnlockMutex(&obj_list_mutex);
-			if (rc != CKR_OK)
-				goto error;
+		if (!fread(&size, sizeof(CK_ULONG_32), 1, fp2)) {
+			fclose(fp2);
+			OCK_SYSLOG(LOG_ERR, "Cannot read size\n");
+			continue;
+		}
+		if (!fread(&priv, sizeof(CK_BBOOL), 1, fp2)) {
+			fclose(fp2);
+			OCK_SYSLOG(LOG_ERR, "Cannot read boolean\n");
+			continue;
+		}
+		if (priv == FALSE) {
+			fclose(fp2);
+			continue;
+		}
+		//size--;
+		size = size - sizeof(CK_ULONG_32) - sizeof(CK_BBOOL);
+		buf = (CK_BYTE *) malloc(size);
+		if (!buf) {
+			fclose(fp2);
+			OCK_SYSLOG(LOG_ERR,
+				   "Cannot malloc %u bytes to read in "
+				   "token object %s (ignoring it)",
+				   size, fname);
+			continue;
+		}
 
+		read_size = fread((char *)buf, 1, size, fp2);
+		if (read_size != size) {
 			free(buf);
 			fclose(fp2);
+			OCK_SYSLOG(LOG_ERR,
+				   "Cannot read token object %s "
+				   "(ignoring it)", fname);
+			continue;
 		}
+		// Grab object list  mutex
+		MY_LockMutex(&obj_list_mutex);
+		rc = restore_private_token_object(tokdata, buf,
+						  size, NULL);
+		MY_UnlockMutex(&obj_list_mutex);
+		if (rc != CKR_OK)
+			goto error;
+
+		free(buf);
+		fclose(fp2);
 	}
 	fclose(fp1);
 
@@ -903,7 +906,8 @@ error:
 
 //
 //
-CK_RV restore_private_token_object(CK_BYTE * data, CK_ULONG len, OBJECT * pObj)
+CK_RV restore_private_token_object(STDLL_TokData_t *tokdata, CK_BYTE * data,
+				   CK_ULONG len, OBJECT * pObj)
 {
 	CK_BYTE *clear = NULL;
 	CK_BYTE *obj_data = NULL;
@@ -942,9 +946,9 @@ CK_RV restore_private_token_object(CK_BYTE * data, CK_ULONG len, OBJECT * pObj)
 		rc = ERR_HOST_MEMORY;
 		goto done;
 	}
-	memcpy(key, master_key, key_len);
+	memcpy(key, tokdata->master_key, key_len);
 
-	rc = decrypt_data_with_clear_key(key, key_len,
+	rc = decrypt_data_with_clear_key(tokdata, key, key_len,
 			  token_specific.data_store.obj_initial_vector,
 			  data, len, clear, &clear_len);
 	if (rc != CKR_OK) {
@@ -978,7 +982,7 @@ CK_RV restore_private_token_object(CK_BYTE * data, CK_ULONG len, OBJECT * pObj)
 
 	// check the hash
 	//
-	rc = compute_sha1(ptr, obj_data_len, hash_sha);
+	rc = compute_sha1(tokdata, ptr, obj_data_len, hash_sha);
 	if (rc != CKR_OK) {
 		goto done;
 	}
@@ -993,7 +997,7 @@ CK_RV restore_private_token_object(CK_BYTE * data, CK_ULONG len, OBJECT * pObj)
 	// the token object...
 	//
 
-	rc = object_mgr_restore_obj(obj_data, pObj);
+	rc = object_mgr_restore_obj(tokdata, obj_data, pObj);
 	if (rc != CKR_OK) {
 		goto done;
 	}
@@ -1010,7 +1014,7 @@ done:
 
 //
 //
-CK_RV load_masterkey_so(void)
+CK_RV load_masterkey_so(STDLL_TokData_t *tokdata)
 {
 	FILE *fp = NULL;
 	CK_BYTE hash_sha[SHA1_HASH_SIZE];
@@ -1024,7 +1028,6 @@ CK_RV load_masterkey_so(void)
 	CK_ULONG key_len = 0L;
 	CK_ULONG master_key_len = 0L;
 	CK_ULONG block_size = 0L;
-	char pk_dir_buf[PATH_MAX];
 
 	if ((rc = get_encryption_info_for_clear_key(&key_len,
 						    &block_size)) != CKR_OK)
@@ -1033,7 +1036,7 @@ CK_RV load_masterkey_so(void)
 	if ((rc = get_encryption_info(&master_key_len, NULL)) != CKR_OK)
 		goto done;
 
-	memset(master_key, 0x0, master_key_len);
+	memset(tokdata->master_key, 0x0, master_key_len);
 
 	data_len = master_key_len + SHA1_HASH_SIZE;
 	clear_len = cipher_len = (data_len + block_size - 1)
@@ -1050,7 +1053,7 @@ CK_RV load_masterkey_so(void)
 	// this file gets created on C_InitToken so we can assume that it always
 	// exists
 	//
-	sprintf(fname, "%s/MK_SO", get_pk_dir(pk_dir_buf));
+	sprintf(fname, "%s/MK_SO", tokdata->data_store);
 	fp = fopen((char *)fname, "r");
 	if (!fp) {
 		TRACE_ERROR("fopen(%s): %s\n", fname, strerror(errno));
@@ -1069,10 +1072,11 @@ CK_RV load_masterkey_so(void)
 	// decrypt the master key data using the MD5 of the SO key
 	// (we can't use the SHA of the SO key since the SHA of the key is
 	// stored in the token data file).
-	memcpy(key, so_pin_md5, MD5_HASH_SIZE);
-	memcpy(key + MD5_HASH_SIZE, so_pin_md5, key_len - MD5_HASH_SIZE);
+	memcpy(key, tokdata->so_pin_md5, MD5_HASH_SIZE);
+	memcpy(key + MD5_HASH_SIZE, tokdata->so_pin_md5,
+	       key_len - MD5_HASH_SIZE);
 
-	rc = decrypt_data_with_clear_key(key, key_len,
+	rc = decrypt_data_with_clear_key(tokdata, key, key_len,
 					 token_specific.data_store.pin_initial_vector,
 					 cipher, cipher_len,
 					 clear, &clear_len);
@@ -1088,7 +1092,7 @@ CK_RV load_masterkey_so(void)
 
 	// compare the hashes
 	//
-	rc = compute_sha1(clear, master_key_len, hash_sha);
+	rc = compute_sha1(tokdata, clear, master_key_len, hash_sha);
 	if (rc != CKR_OK) {
 		goto done;
 	}
@@ -1099,7 +1103,7 @@ CK_RV load_masterkey_so(void)
 		goto done;
 	}
 
-	memcpy(master_key, clear, master_key_len);
+	memcpy(tokdata->master_key, clear, master_key_len);
 	rc = CKR_OK;
 
 done:
@@ -1116,7 +1120,7 @@ done:
 
 //
 //
-CK_RV load_masterkey_user(void)
+CK_RV load_masterkey_user(STDLL_TokData_t *tokdata)
 {
 	FILE *fp = NULL;
 	CK_BYTE hash_sha[SHA1_HASH_SIZE];
@@ -1130,7 +1134,6 @@ CK_RV load_masterkey_user(void)
 	CK_ULONG key_len = 0L;
 	CK_ULONG master_key_len = 0L;
 	CK_ULONG block_size = 0L;
-	char pk_dir_buf[PATH_MAX];
 
 	if ((rc = get_encryption_info_for_clear_key(&key_len,
 						    &block_size)) != CKR_OK)
@@ -1139,7 +1142,7 @@ CK_RV load_masterkey_user(void)
 	if ((rc = get_encryption_info(&master_key_len, NULL)) != CKR_OK)
 		goto done;
 
-	memset(master_key, 0x0, master_key_len);
+	memset(tokdata->master_key, 0x0, master_key_len);
 
 	data_len = master_key_len + SHA1_HASH_SIZE;
 	clear_len = cipher_len = (data_len + block_size - 1)
@@ -1156,7 +1159,7 @@ CK_RV load_masterkey_user(void)
 	// this file gets created on C_InitToken so we can assume that it always
 	// exists
 	//
-	sprintf(fname, "%s/MK_USER", get_pk_dir(pk_dir_buf));
+	sprintf(fname, "%s/MK_USER", tokdata->data_store);
 	fp = fopen((char *)fname, "r");
 	if (!fp) {
 		TRACE_ERROR("fopen(%s): %s\n", fname, strerror(errno));
@@ -1174,10 +1177,11 @@ CK_RV load_masterkey_user(void)
 	// decrypt the master key data using the MD5 of the SO key
 	// (we can't use the SHA of the SO key since the SHA of the key is
 	// stored in the token data file).
-	memcpy(key, user_pin_md5, MD5_HASH_SIZE);
-	memcpy(key + MD5_HASH_SIZE, user_pin_md5, key_len - MD5_HASH_SIZE);
+	memcpy(key, tokdata->user_pin_md5, MD5_HASH_SIZE);
+	memcpy(key + MD5_HASH_SIZE, tokdata->user_pin_md5,
+	       key_len - MD5_HASH_SIZE);
 
-	rc = decrypt_data_with_clear_key(key, key_len,
+	rc = decrypt_data_with_clear_key(tokdata, key, key_len,
 					 token_specific.data_store.pin_initial_vector,
 					 cipher, cipher_len,
 					 clear, &clear_len);
@@ -1193,7 +1197,7 @@ CK_RV load_masterkey_user(void)
 
 	// compare the hashes
 	//
-	rc = compute_sha1(clear, master_key_len, hash_sha);
+	rc = compute_sha1(tokdata, clear, master_key_len, hash_sha);
 	if (rc != CKR_OK) {
 		goto done;
 	}
@@ -1204,7 +1208,7 @@ CK_RV load_masterkey_user(void)
 		goto done;
 	}
 
-	memcpy(master_key, clear, master_key_len);
+	memcpy(tokdata->master_key, clear, master_key_len);
 	rc = CKR_OK;
 
 done:
@@ -1221,7 +1225,7 @@ done:
 
 //
 //
-CK_RV save_masterkey_so(void)
+CK_RV save_masterkey_so(STDLL_TokData_t *tokdata)
 {
 	FILE *fp = NULL;
 	CK_BYTE *clear = NULL;
@@ -1235,10 +1239,9 @@ CK_RV save_masterkey_so(void)
 	CK_ULONG data_len = 0L;
 	CK_BYTE fname[PATH_MAX];
 	CK_RV rc;
-	char pk_dir_buf[PATH_MAX];
 
 	/* Skip it if master key is not needed. */
-   	if (!token_specific.data_store.use_master_key)
+	if (!token_specific.data_store.use_master_key)
 		return CKR_OK;
 
 	if ((rc = get_encryption_info_for_clear_key(&key_len,
@@ -1261,18 +1264,19 @@ CK_RV save_masterkey_so(void)
 	}
 
 	// Copy data to buffer (key+hash)
-	memcpy(clear, master_key, master_key_len);
-	if ((rc = compute_sha1(master_key, master_key_len,
-			      clear + master_key_len)) != CKR_OK)
+	memcpy(clear, tokdata->master_key, master_key_len);
+	if ((rc = compute_sha1(tokdata, tokdata->master_key,
+			       master_key_len, clear + master_key_len)) != CKR_OK)
 		goto done;
 	add_pkcs_padding(clear + data_len, block_size, data_len,
 			 clear_len);
 
 	// encrypt the key data
-	memcpy(key, so_pin_md5, MD5_HASH_SIZE);
-	memcpy(key + MD5_HASH_SIZE, so_pin_md5, key_len - MD5_HASH_SIZE);
+	memcpy(key, tokdata->so_pin_md5, MD5_HASH_SIZE);
+	memcpy(key + MD5_HASH_SIZE, tokdata->so_pin_md5,
+	       key_len - MD5_HASH_SIZE);
 
-	rc = encrypt_data_with_clear_key(key, key_len,
+	rc = encrypt_data_with_clear_key(tokdata, key, key_len,
 					 token_specific.data_store.pin_initial_vector,
 					 clear, clear_len,
 					 cipher, &cipher_len);
@@ -1284,7 +1288,7 @@ CK_RV save_masterkey_so(void)
 	//
 	// probably ought to ensure the permissions are correct
 	//
-	sprintf(fname, "%s/MK_SO", get_pk_dir(pk_dir_buf));
+	sprintf(fname, "%s/MK_SO", tokdata->data_store);
 	fp = fopen((char *)fname, "w");
 	if (!fp) {
 		TRACE_ERROR("fopen(%s): %s\n", fname, strerror(errno));
@@ -1316,7 +1320,7 @@ done:
 
 //
 //
-CK_RV save_masterkey_user(void)
+CK_RV save_masterkey_user(STDLL_TokData_t *tokdata)
 {
 	FILE *fp = NULL;
 	CK_BYTE *clear = NULL;
@@ -1330,7 +1334,6 @@ CK_RV save_masterkey_user(void)
 	CK_ULONG data_len = 0L;
 	CK_BYTE fname[PATH_MAX];
 	CK_RV rc;
-	char pk_dir_buf[PATH_MAX];
 
 	if ((rc = get_encryption_info_for_clear_key(&key_len,
 						    &block_size)) != CKR_OK)
@@ -1352,18 +1355,19 @@ CK_RV save_masterkey_user(void)
 	}
 
 	// Copy data to buffer (key+hash)
-	memcpy(clear, master_key, master_key_len);
-	if ((rc = compute_sha1(master_key, master_key_len,
-			      clear + master_key_len)) != CKR_OK)
+	memcpy(clear, tokdata->master_key, master_key_len);
+	if ((rc = compute_sha1(tokdata, tokdata->master_key,
+			       master_key_len, clear + master_key_len)) != CKR_OK)
 		goto done;
 	add_pkcs_padding(clear + data_len, block_size , data_len,
 			 clear_len);
 
 	// encrypt the key data
-	memcpy(key, user_pin_md5, MD5_HASH_SIZE);
-	memcpy(key + MD5_HASH_SIZE, user_pin_md5, key_len - MD5_HASH_SIZE);
+	memcpy(key, tokdata->user_pin_md5, MD5_HASH_SIZE);
+	memcpy(key + MD5_HASH_SIZE, tokdata->user_pin_md5,
+	       key_len - MD5_HASH_SIZE);
 
-	rc = encrypt_data_with_clear_key(key, key_len,
+	rc = encrypt_data_with_clear_key(tokdata, key, key_len,
 					 token_specific.data_store.pin_initial_vector,
 					 clear, clear_len,
 					 cipher, &cipher_len);
@@ -1375,7 +1379,7 @@ CK_RV save_masterkey_user(void)
 	//
 	// probably ought to ensure the permissions are correct
 	//
-	sprintf(fname, "%s/MK_USER", get_pk_dir(pk_dir_buf));
+	sprintf(fname, "%s/MK_USER", tokdata->data_store);
 	fp = fopen((char *)fname, "w");
 	if (!fp) {
 		TRACE_ERROR("fopen(%s): %s\n", fname, strerror(errno));
@@ -1407,7 +1411,7 @@ done:
 
 //
 //
-CK_RV reload_token_object(OBJECT * obj)
+CK_RV reload_token_object(STDLL_TokData_t *tokdata, OBJECT * obj)
 {
 	FILE *fp = NULL;
 	CK_BYTE *buf = NULL;
@@ -1417,12 +1421,10 @@ CK_RV reload_token_object(OBJECT * obj)
 	CK_ULONG size_64;
 	CK_RV rc;
 	size_t read_size;
-	char pk_dir_buf[PATH_MAX];
 
 	memset((char *)fname, 0x0, sizeof(fname));
 
-	sprintf((char *)fname, "%s/%s/", get_pk_dir(pk_dir_buf),
-		PK_LITE_OBJ_DIR);
+	sprintf((char *)fname, "%s/%s/", tokdata->data_store, PK_LITE_OBJ_DIR);
 
 	strncat((char *)fname, (char *)obj->name, 8);
 
@@ -1435,8 +1437,17 @@ CK_RV reload_token_object(OBJECT * obj)
 
 	set_perm(fileno(fp));
 
-	fread(&size, sizeof(CK_ULONG_32), 1, fp);
-	fread(&priv, sizeof(CK_BBOOL), 1, fp);
+	if (!fread(&size, sizeof(CK_ULONG_32), 1, fp)) {
+		OCK_SYSLOG(LOG_ERR, "Cannot read size\n");
+		rc = CKR_FUNCTION_FAILED;
+		goto done;
+	}
+
+	if (!fread(&priv, sizeof(CK_BBOOL), 1, fp)) {
+		OCK_SYSLOG(LOG_ERR, "Cannot read boolean\n");
+		rc = CKR_FUNCTION_FAILED;
+		goto done;
+	}
 
 	size = size - sizeof(CK_ULONG_32) - sizeof(CK_BBOOL);	// SAB
 
@@ -1461,9 +1472,9 @@ CK_RV reload_token_object(OBJECT * obj)
 	size_64 = size;
 
 	if (priv)
-		rc = restore_private_token_object(buf, size_64, obj);
+		rc = restore_private_token_object(tokdata, buf, size_64, obj);
 	else
-		rc = object_mgr_restore_obj(buf, obj);
+		rc = object_mgr_restore_obj(tokdata, buf, obj);
 
 done:
 	if (fp)
@@ -1477,16 +1488,15 @@ extern void set_perm(int);
 
 //
 //
-CK_RV delete_token_object(OBJECT * obj)
+CK_RV delete_token_object(STDLL_TokData_t *tokdata, OBJECT * obj)
 {
 	FILE *fp1, *fp2;
 	CK_BYTE line[100];
 	CK_BYTE objidx[PATH_MAX], idxtmp[PATH_MAX], fname[PATH_MAX];
-	char pk_dir_buf[PATH_MAX];
 
-	sprintf((char *)objidx, "%s/%s/%s", get_pk_dir(pk_dir_buf),
+	sprintf((char *)objidx, "%s/%s/%s", tokdata->data_store,
 		PK_LITE_OBJ_DIR, PK_LITE_OBJ_IDX);
-	sprintf((char *)idxtmp, "%s/%s/%s", get_pk_dir(pk_dir_buf),
+	sprintf((char *)idxtmp, "%s/%s/%s", tokdata->data_store,
 		PK_LITE_OBJ_DIR, "IDX.TMP");
 
 	// FIXME:  on UNIX, we need to make sure these guys aren't symlinks
@@ -1509,15 +1519,12 @@ CK_RV delete_token_object(OBJECT * obj)
 
 	set_perm(fileno(fp2));
 
-	while (!feof(fp1)) {
-		(void)fgets((char *)line, 50, fp1);
-		if (!feof(fp1)) {
-			line[strlen((char *)line) - 1] = 0;
-			if (strcmp((char *)line, (char *)obj->name) == 0)
-				continue;
-			else
-				fprintf(fp2, "%s\n", line);
-		}
+	while (fgets((char *)line, 50, fp1)) {
+		line[strlen((char *)line) - 1] = 0;
+		if (strcmp((char *)line, (char *)obj->name) == 0)
+			continue;
+		else
+			fprintf(fp2, "%s\n", line);
 	}
 
 	fclose(fp1);
@@ -1535,27 +1542,24 @@ CK_RV delete_token_object(OBJECT * obj)
 
 	set_perm(fileno(fp2));
 
-	while (!feof(fp1)) {
-		(void)fgets((char *)line, 50, fp1);
-		if (!feof(fp1))
-			fprintf(fp2, "%s", (char *)line);
+	while (fgets((char *)line, 50, fp1)) {
+		fprintf(fp2, "%s", (char *)line);
 	}
 
 	fclose(fp1);
 	fclose(fp2);
 
-	sprintf((char *)fname, "%s/%s/%s", get_pk_dir(pk_dir_buf),
+	sprintf((char *)fname, "%s/%s/%s", tokdata->data_store,
 		PK_LITE_OBJ_DIR, (char *)obj->name);
 	unlink((char *)fname);
 	return CKR_OK;
 
 }
 
-CK_RV delete_token_data()
+CK_RV delete_token_data(STDLL_TokData_t *tokdata)
 {
 	CK_RV rc = CKR_OK;
 	char *cmd = NULL;
-	char pk_dir_buf[PATH_MAX];
 
 	// Construct a string to delete the token objects.
 	//
@@ -1564,18 +1568,20 @@ CK_RV delete_token_data()
 	//
 	// TODO: Implement delete_all_files_in_dir() */
 	if (asprintf(&cmd, "%s %s/%s/* > /dev/null 2>&1", DEL_CMD,
-			    get_pk_dir(pk_dir_buf), PK_LITE_OBJ_DIR) < 0) {
+		     tokdata->data_store, PK_LITE_OBJ_DIR) < 0) {
 		rc = CKR_HOST_MEMORY;
 		goto done;
 	}
 
-	system(cmd);
+	if (system(cmd))
+		TRACE_ERROR("system() failed.\n");
+
 done:
 	free(cmd);
 	return rc;
 }
 
-CK_RV generate_master_key(CK_BYTE *key)
+CK_RV generate_master_key(STDLL_TokData_t *tokdata, CK_BYTE *key)
 {
 	CK_RV rc = CKR_OK;
 	CK_ULONG key_len = 0L;
@@ -1593,33 +1599,44 @@ CK_RV generate_master_key(CK_BYTE *key)
 	 * software(openssl), not token. So generate masterkey via RNG.
 	 */
 	if (token_specific.token_keysize)
-		return rng_generate(key, key_len);
+		return rng_generate(tokdata, key, key_len);
 
 	/* For clear key tokens, let token generate masterkey
 	 * since token will also encrypt/decrypt the objects.
 	 */
 	switch (token_specific.data_store.encryption_algorithm) {
 	case CKM_DES3_CBC:
-		return token_specific.t_des_key_gen(key, master_key_len,
-						    key_len);
+		return token_specific.t_des_key_gen(tokdata, key,
+						    master_key_len, key_len);
 	case CKM_AES_CBC:
-		return token_specific.t_aes_key_gen(key, master_key_len,
-						    key_len);
+		return token_specific.t_aes_key_gen(tokdata, key,
+						    master_key_len, key_len);
 	}
 
 	return ERR_MECHANISM_INVALID;
 }
 
-void init_data_store(char *directory)
+void init_data_store(char *directory, char *data_store)
 {
 	char *pkdir;
 	if ((pkdir = getenv("PKCS_APP_STORE")) != NULL) {
 		pk_dir = (char *)malloc(strlen(pkdir) + 1024);
 		memset(pk_dir, 0, strlen(pkdir) + 1024);
 		sprintf(pk_dir, "%s/%s", pkdir, SUB_DIR);
-	} else {
+		return;
+	}
+
+	if (directory) {
 		pk_dir = (char *)malloc(strlen(directory) + 25);
 		memset(pk_dir, 0, strlen(directory) + 25);
 		sprintf(pk_dir, "%s", directory);
+		memcpy(data_store, pk_dir, strlen(directory) + 25);
 	}
+	else {
+		pk_dir = (char *)malloc(strlen(PK_DIR) + 25);
+		memset(pk_dir, 0, strlen(PK_DIR) + 25);
+		sprintf(pk_dir, "%s", PK_DIR);
+		memcpy(data_store, pk_dir, strlen(PK_DIR) + 25);
+	}
+	return;
 }
