@@ -100,18 +100,19 @@ CK_RV get_ecsiglen(OBJECT *key_obj, CK_ULONG *size)
 }
 
 CK_RV
-ckm_ec_key_pair_gen( TEMPLATE  * publ_tmpl,
-		TEMPLATE  * priv_tmpl )
+ckm_ec_key_pair_gen( STDLL_TokData_t *tokdata, TEMPLATE  * publ_tmpl,
+		     TEMPLATE  * priv_tmpl )
 {
 	CK_RV rc;
-	rc = token_specific.t_ec_generate_keypair(publ_tmpl, priv_tmpl);
+	rc = token_specific.t_ec_generate_keypair(tokdata, publ_tmpl, priv_tmpl);
 	if (rc != CKR_OK)
 		TRACE_ERROR("Key Generation failed\n");
 	return rc;
 }
 
 CK_RV
-ckm_ec_sign( CK_BYTE		*in_data,
+ckm_ec_sign(	STDLL_TokData_t	*tokdata,
+		CK_BYTE		*in_data,
 		CK_ULONG	in_data_len,
 		CK_BYTE		*out_data,
 		CK_ULONG	*out_data_len,
@@ -135,8 +136,8 @@ ckm_ec_sign( CK_BYTE		*in_data,
 		TRACE_ERROR("This operation requires a private key.\n");
 		return CKR_KEY_FUNCTION_NOT_PERMITTED;
 	}
-	rc = token_specific.t_ec_sign(in_data, in_data_len, out_data,
-					out_data_len, key_obj);
+	rc = token_specific.t_ec_sign(tokdata, in_data, in_data_len, out_data,
+				      out_data_len, key_obj);
 	if (rc != CKR_OK)
 		TRACE_DEVEL("EC Sign failed.\n");
 
@@ -144,13 +145,14 @@ ckm_ec_sign( CK_BYTE		*in_data,
 }
 
 CK_RV
-ec_sign( SESSION			*sess,
-	       CK_BBOOL			length_only,
-	       SIGN_VERIFY_CONTEXT	*ctx,
-	       CK_BYTE			*in_data,
-	       CK_ULONG			in_data_len,
-	       CK_BYTE			*out_data,
-	       CK_ULONG			*out_data_len )
+ec_sign( STDLL_TokData_t		*tokdata,
+	 SESSION			*sess,
+	 CK_BBOOL			length_only,
+	 SIGN_VERIFY_CONTEXT		*ctx,
+	 CK_BYTE			*in_data,
+	 CK_ULONG			in_data_len,
+	 CK_BYTE			*out_data,
+	 CK_ULONG			*out_data_len )
 {
 	OBJECT          *key_obj   = NULL;
 	CK_ULONG         plen;
@@ -161,7 +163,7 @@ ec_sign( SESSION			*sess,
 		return CKR_FUNCTION_FAILED;
 	}
 
-	rc = object_mgr_find_in_map1( ctx->key, &key_obj );
+	rc = object_mgr_find_in_map1( tokdata, ctx->key, &key_obj );
 	if (rc != CKR_OK){
 		TRACE_ERROR("Failed to acquire key from specified handle");
 		if (rc == CKR_OBJECT_HANDLE_INVALID)
@@ -186,13 +188,14 @@ ec_sign( SESSION			*sess,
 		return CKR_BUFFER_TOO_SMALL;
 	}
 
-	rc = ckm_ec_sign( in_data, in_data_len, out_data,
+	rc = ckm_ec_sign( tokdata, in_data, in_data_len, out_data,
 			out_data_len, key_obj );
 	return rc;
 }
 
 CK_RV
-ckm_ec_verify( CK_BYTE		*in_data,
+ckm_ec_verify(  STDLL_TokData_t *tokdata,
+		CK_BYTE		*in_data,
 		CK_ULONG	in_data_len,
 		CK_BYTE		*out_data,
 		CK_ULONG	out_data_len,
@@ -217,8 +220,8 @@ ckm_ec_verify( CK_BYTE		*in_data,
 		return CKR_KEY_FUNCTION_NOT_PERMITTED;
 	}
 
-	rc = token_specific.t_ec_verify(in_data, in_data_len,
-			out_data, out_data_len, key_obj);
+	rc = token_specific.t_ec_verify(tokdata, in_data, in_data_len,
+					out_data, out_data_len, key_obj);
 	if (rc != CKR_OK)
 		TRACE_ERROR("Token specific ec verify failed.\n");
 
@@ -226,7 +229,8 @@ ckm_ec_verify( CK_BYTE		*in_data,
 }
 
 CK_RV
-ec_verify(SESSION		*sess,
+ec_verify(STDLL_TokData_t	*tokdata,
+	SESSION			*sess,
 	SIGN_VERIFY_CONTEXT	*ctx,
 	CK_BYTE			*in_data,
 	CK_ULONG		in_data_len,
@@ -238,7 +242,7 @@ ec_verify(SESSION		*sess,
 	CK_RV            rc;
 
 
-	rc = object_mgr_find_in_map1(ctx->key, &key_obj);
+	rc = object_mgr_find_in_map1(tokdata, ctx->key, &key_obj);
 	if (rc != CKR_OK){
 		TRACE_ERROR("Failed to acquire key from specified handle");
 		if (rc == CKR_OBJECT_HANDLE_INVALID)
@@ -259,14 +263,15 @@ ec_verify(SESSION		*sess,
 		TRACE_ERROR("%s\n", ock_err(ERR_SIGNATURE_LEN_RANGE));
 		return CKR_SIGNATURE_LEN_RANGE;
 	}
-	rc = ckm_ec_verify(in_data, in_data_len, signature,
+	rc = ckm_ec_verify(tokdata, in_data, in_data_len, signature,
 			sig_len, key_obj);
 
 	return rc;
 }
 
 CK_RV
-ec_hash_sign( SESSION              * sess,
+ec_hash_sign(	STDLL_TokData_t      * tokdata,
+		SESSION              * sess,
 		CK_BBOOL               length_only,
 		SIGN_VERIFY_CONTEXT  * ctx,
 		CK_BYTE              * in_data,
@@ -315,13 +320,14 @@ ec_hash_sign( SESSION              * sess,
       return rc;
    }
 
-   rc = digest_mgr_init( sess, &digest_ctx, &digest_mech );
+   rc = digest_mgr_init( tokdata, sess, &digest_ctx, &digest_mech );
    if (rc != CKR_OK){
       TRACE_DEVEL("Digest Mgr Init failed.\n");
       return rc;
    }
 
-   rc = digest_mgr_digest( sess, length_only, &digest_ctx, in_data, in_data_len, hash, &hash_len );
+   rc = digest_mgr_digest( tokdata, sess, length_only, &digest_ctx, in_data,
+			   in_data_len, hash, &hash_len );
    if (rc != CKR_OK){
       TRACE_DEVEL("Digest Mgr Digest failed.\n");
       return rc;
@@ -331,13 +337,14 @@ ec_hash_sign( SESSION              * sess,
     sign_mech.ulParameterLen = 0;
     sign_mech.pParameter     = NULL;
 
-   rc = sign_mgr_init( sess, &sign_ctx, &sign_mech, FALSE, ctx->key );
+   rc = sign_mgr_init( tokdata, sess, &sign_ctx, &sign_mech, FALSE, ctx->key );
    if (rc != CKR_OK){
       TRACE_DEVEL("Sign Mgr Init failed.\n");
       goto error;
    }
 
-   rc = sign_mgr_sign( sess, length_only, &sign_ctx, hash, hash_len, signature, sig_len );
+   rc = sign_mgr_sign( tokdata, sess, length_only, &sign_ctx, hash, hash_len,
+		       signature, sig_len );
    if (rc != CKR_OK)
       TRACE_DEVEL("Sign Mgr Sign failed.\n");
 
@@ -347,10 +354,11 @@ error:
 }
 
 CK_RV
-ec_hash_sign_update( SESSION              * sess,
-		     SIGN_VERIFY_CONTEXT  * ctx,
-		     CK_BYTE              * in_data,
-		     CK_ULONG               in_data_len )
+ec_hash_sign_update( STDLL_TokData_t      * tokdata,
+		     SESSION              * sess,
+                     SIGN_VERIFY_CONTEXT  * ctx,
+                     CK_BYTE              * in_data,
+                     CK_ULONG               in_data_len )
 {
    RSA_DIGEST_CONTEXT  * context = NULL;
    CK_MECHANISM          digest_mech;
@@ -383,7 +391,8 @@ ec_hash_sign_update( SESSION              * sess,
       digest_mech.ulParameterLen = 0;
       digest_mech.pParameter     = NULL;
 
-      rc = digest_mgr_init( sess, &context->hash_context, &digest_mech );
+      rc = digest_mgr_init( tokdata, sess, &context->hash_context,
+			    &digest_mech );
       if (rc != CKR_OK){
 	 TRACE_DEVEL("Digest Mgr Init failed.\n");
 	 return rc;
@@ -391,7 +400,8 @@ ec_hash_sign_update( SESSION              * sess,
       context->flag = TRUE;
    }
 
-   rc = digest_mgr_digest_update( sess, &context->hash_context, in_data, in_data_len );
+   rc = digest_mgr_digest_update( tokdata, sess, &context->hash_context, in_data,
+				  in_data_len );
    if (rc != CKR_OK){
       TRACE_DEVEL("Digest Mgr Update failed.\n");
       return rc;
@@ -400,11 +410,12 @@ ec_hash_sign_update( SESSION              * sess,
 }
 
 CK_RV
-ec_hash_sign_final( SESSION              * sess,
-		    CK_BBOOL               length_only,
-		    SIGN_VERIFY_CONTEXT  * ctx,
-		    CK_BYTE              * signature,
-		    CK_ULONG             * sig_len )
+ec_hash_sign_final( STDLL_TokData_t      * tokdata,
+		    SESSION              * sess,
+                    CK_BBOOL               length_only,
+                    SIGN_VERIFY_CONTEXT  * ctx,
+                    CK_BYTE              * signature,
+                    CK_ULONG             * sig_len )
 {
    CK_BYTE               hash[MAX_SHA_HASH_SIZE];
    RSA_DIGEST_CONTEXT  * context = NULL;
@@ -428,7 +439,8 @@ ec_hash_sign_final( SESSION              * sess,
       return rc;
    }
 
-   rc = digest_mgr_digest_final( sess, length_only, &context->hash_context, hash, &hash_len );
+   rc = digest_mgr_digest_final( tokdata, sess, length_only,
+				 &context->hash_context, hash, &hash_len );
    if (rc != CKR_OK){
       TRACE_DEVEL("Digest Mgr Final failed.\n");
       return rc;
@@ -438,14 +450,14 @@ ec_hash_sign_final( SESSION              * sess,
    sign_mech.ulParameterLen = 0;
    sign_mech.pParameter     = NULL;
 
-   rc = sign_mgr_init( sess, &sign_ctx, &sign_mech, FALSE, ctx->key );
+   rc = sign_mgr_init( tokdata, sess, &sign_ctx, &sign_mech, FALSE, ctx->key );
    if (rc != CKR_OK){
       TRACE_DEVEL("Sign Mgr Init failed.\n");
       goto done;
    }
 
    //rc = sign_mgr_sign( sess, length_only, &sign_ctx, ber_data, ber_data_len, signature, sig_len );
-   rc = sign_mgr_sign( sess, length_only, &sign_ctx, hash, hash_len, signature, sig_len );
+   rc = sign_mgr_sign( tokdata, sess, length_only, &sign_ctx, hash, hash_len, signature, sig_len );
    if (rc != CKR_OK)
       TRACE_DEVEL("Sign Mgr Sign failed.\n");
 
@@ -460,12 +472,13 @@ done:
 }
 
 CK_RV
-ec_hash_verify( SESSION              * sess,
-		SIGN_VERIFY_CONTEXT  * ctx,
-		CK_BYTE              * in_data,
-		CK_ULONG               in_data_len,
-		CK_BYTE              * signature,
-		CK_ULONG               sig_len )
+ec_hash_verify( STDLL_TokData_t      * tokdata,
+		SESSION              * sess,
+                SIGN_VERIFY_CONTEXT  * ctx,
+                CK_BYTE              * in_data,
+                CK_ULONG               in_data_len,
+                CK_BYTE              * signature,
+                CK_ULONG               sig_len )
 {
    CK_BYTE              hash[MAX_SHA_HASH_SIZE];
    DIGEST_CONTEXT       digest_ctx;
@@ -508,13 +521,14 @@ ec_hash_verify( SESSION              * sess,
       return rc;
    }
 
-   rc = digest_mgr_init( sess, &digest_ctx, &digest_mech );
+   rc = digest_mgr_init( tokdata, sess, &digest_ctx, &digest_mech );
    if (rc != CKR_OK){
       TRACE_DEVEL("Digest Mgr Init failed.\n");
       return rc;
    }
 
-   rc = digest_mgr_digest( sess, FALSE, &digest_ctx, in_data, in_data_len, hash, &hash_len );
+   rc = digest_mgr_digest( tokdata, sess, FALSE, &digest_ctx, in_data,
+			   in_data_len, hash, &hash_len );
    if (rc != CKR_OK){
       TRACE_DEVEL("Digest Mgr Digest failed.\n");
       return rc;
@@ -526,14 +540,14 @@ ec_hash_verify( SESSION              * sess,
    verify_mech.ulParameterLen = 0;
    verify_mech.pParameter     = NULL;
 
-   rc = verify_mgr_init( sess, &verify_ctx, &verify_mech, FALSE, ctx->key );
+   rc = verify_mgr_init( tokdata, sess, &verify_ctx, &verify_mech, FALSE, ctx->key );
    if (rc != CKR_OK){
       TRACE_DEVEL("Verify Mgr Init failed.\n");
       goto done;
    }
 
    //rc = verify_mgr_verify( sess, &verify_ctx, ber_data, ber_data_len, signature, sig_len );
-   rc = verify_mgr_verify( sess, &verify_ctx, hash, hash_len, signature, sig_len );
+   rc = verify_mgr_verify( tokdata, sess, &verify_ctx, hash, hash_len, signature, sig_len );
    if (rc != CKR_OK)
       TRACE_DEVEL("Verify Mgr Verify failed.\n");
 done:
@@ -543,10 +557,11 @@ done:
 
 
 CK_RV
-ec_hash_verify_update( SESSION              * sess,
-		       SIGN_VERIFY_CONTEXT  * ctx,
-		       CK_BYTE              * in_data,
-		       CK_ULONG               in_data_len )
+ec_hash_verify_update( STDLL_TokData_t      * tokdata,
+		       SESSION              * sess,
+                       SIGN_VERIFY_CONTEXT  * ctx,
+                       CK_BYTE              * in_data,
+                       CK_ULONG               in_data_len )
 {
    RSA_DIGEST_CONTEXT  * context = NULL;
    CK_MECHANISM          digest_mech;
@@ -579,7 +594,8 @@ ec_hash_verify_update( SESSION              * sess,
       digest_mech.ulParameterLen = 0;
       digest_mech.pParameter     = NULL;
 
-      rc = digest_mgr_init( sess, &context->hash_context, &digest_mech );
+      rc = digest_mgr_init( tokdata, sess, &context->hash_context,
+			    &digest_mech );
       if (rc != CKR_OK){
 	 TRACE_DEVEL("Digest Mgr Init failed.\n");
 	 return rc;
@@ -587,7 +603,8 @@ ec_hash_verify_update( SESSION              * sess,
       context->flag = TRUE;
    }
 
-   rc = digest_mgr_digest_update( sess, &context->hash_context, in_data, in_data_len );
+   rc = digest_mgr_digest_update( tokdata, sess, &context->hash_context,
+				  in_data, in_data_len );
    if (rc != CKR_OK){
       TRACE_DEVEL("Digest Mgr Update failed.\n");
       return rc;
@@ -596,10 +613,11 @@ ec_hash_verify_update( SESSION              * sess,
 }
 
 CK_RV
-ec_hash_verify_final( SESSION              * sess,
-		      SIGN_VERIFY_CONTEXT  * ctx,
-		      CK_BYTE              * signature,
-		      CK_ULONG               sig_len )
+ec_hash_verify_final( STDLL_TokData_t      * tokdata,
+		      SESSION              * sess,
+                      SIGN_VERIFY_CONTEXT  * ctx,
+                      CK_BYTE              * signature,
+                      CK_ULONG               sig_len )
 {
    CK_BYTE               hash[MAX_SHA_HASH_SIZE];
    RSA_DIGEST_CONTEXT  * context = NULL;
@@ -622,7 +640,8 @@ ec_hash_verify_final( SESSION              * sess,
       return rc;
    }
 
-   rc = digest_mgr_digest_final( sess, FALSE, &context->hash_context, hash, &hash_len );
+   rc = digest_mgr_digest_final( tokdata, sess, FALSE, &context->hash_context,
+				 hash, &hash_len );
    if (rc != CKR_OK){
       TRACE_DEVEL("Digest Mgr Final failed.\n");
       return rc;
@@ -631,13 +650,13 @@ ec_hash_verify_final( SESSION              * sess,
    verify_mech.ulParameterLen = 0;
    verify_mech.pParameter     = NULL;
 
-   rc = verify_mgr_init( sess, &verify_ctx, &verify_mech, FALSE, ctx->key );
+   rc = verify_mgr_init( tokdata, sess, &verify_ctx, &verify_mech, FALSE, ctx->key );
    if (rc != CKR_OK){
       TRACE_DEVEL("Verify Mgr Init failed.\n");
       goto done;
    }
 
-   rc = verify_mgr_verify( sess, &verify_ctx, hash, hash_len, signature, sig_len );
+   rc = verify_mgr_verify( tokdata, sess, &verify_ctx, hash, hash_len, signature, sig_len );
    if (rc != CKR_OK)
       TRACE_DEVEL("Verify Mgr Verify failed.\n");
 done:
