@@ -1,293 +1,12 @@
 /*
-             Common Public License Version 0.5
-
-             THE ACCOMPANYING PROGRAM IS PROVIDED UNDER THE TERMS OF
-             THIS COMMON PUBLIC LICENSE ("AGREEMENT"). ANY USE,
-             REPRODUCTION OR DISTRIBUTION OF THE PROGRAM CONSTITUTES
-             RECIPIENT'S ACCEPTANCE OF THIS AGREEMENT.
-
-             1. DEFINITIONS
-
-             "Contribution" means: 
-                   a) in the case of the initial Contributor, the
-                   initial code and documentation distributed under
-                   this Agreement, and 
-
-                   b) in the case of each subsequent Contributor:
-                   i) changes to the Program, and
-                   ii) additions to the Program;
-
-                   where such changes and/or additions to the Program
-                   originate from and are distributed by that
-                   particular Contributor. A Contribution 'originates'
-                   from a Contributor if it was added to the Program
-                   by such Contributor itself or anyone acting on such
-                   Contributor's behalf. Contributions do not include
-                   additions to the Program which: (i) are separate
-                   modules of software distributed in conjunction with
-                   the Program under their own license agreement, and
-                   (ii) are not derivative works of the Program.
-
-
-             "Contributor" means any person or entity that distributes
-             the Program.
-
-             "Licensed Patents " mean patent claims licensable by a
-             Contributor which are necessarily infringed by the use or
-             sale of its Contribution alone or when combined with the
-             Program. 
-
-             "Program" means the Contributions distributed in
-             accordance with this Agreement.
-
-             "Recipient" means anyone who receives the Program under
-             this Agreement, including all Contributors.
-
-             2. GRANT OF RIGHTS
-
-                   a) Subject to the terms of this Agreement, each
-                   Contributor hereby grants Recipient a
-                   non-exclusive, worldwide, royalty-free copyright
-                   license to reproduce, prepare derivative works of,
-                   publicly display, publicly perform, distribute and
-                   sublicense the Contribution of such Contributor, if
-                   any, and such derivative works, in source code and
-                   object code form.
-
-                   b) Subject to the terms of this Agreement, each
-                   Contributor hereby grants Recipient a
-                   non-exclusive, worldwide, royalty-free patent
-                   license under Licensed Patents to make, use, sell,
-                   offer to sell, import and otherwise transfer the
-                   Contribution of such Contributor, if any, in source
-                   code and object code form. This patent license
-                   shall apply to the combination of the Contribution
-                   and the Program if, at the time the Contribution is
-                   added by the Contributor, such addition of the
-                   Contribution causes such combination to be covered
-                   by the Licensed Patents. The patent license shall
-                   not apply to any other combinations which include
-                   the Contribution. No hardware per se is licensed
-                   hereunder.
-
-                   c) Recipient understands that although each
-                   Contributor grants the licenses to its
-                   Contributions set forth herein, no assurances are
-                   provided by any Contributor that the Program does
-                   not infringe the patent or other intellectual
-                   property rights of any other entity. Each
-                   Contributor disclaims any liability to Recipient
-                   for claims brought by any other entity based on
-                   infringement of intellectual property rights or
-                   otherwise. As a condition to exercising the rights
-                   and licenses granted hereunder, each Recipient
-                   hereby assumes sole responsibility to secure any
-                   other intellectual property rights needed, if any.
-
-                   For example, if a third party patent license is
-                   required to allow Recipient to distribute the
-                   Program, it is Recipient's responsibility to
-                   acquire that license before distributing the
-                   Program.
-
-                   d) Each Contributor represents that to its
-                   knowledge it has sufficient copyright rights in its
-                   Contribution, if any, to grant the copyright
-                   license set forth in this Agreement.
-
-             3. REQUIREMENTS
-
-             A Contributor may choose to distribute the Program in
-             object code form under its own license agreement, provided
-             that:
-                   a) it complies with the terms and conditions of
-                   this Agreement; and
-
-                   b) its license agreement:
-                   i) effectively disclaims on behalf of all
-                   Contributors all warranties and conditions, express
-                   and implied, including warranties or conditions of
-                   title and non-infringement, and implied warranties
-                   or conditions of merchantability and fitness for a
-                   particular purpose;
-
-                   ii) effectively excludes on behalf of all
-                   Contributors all liability for damages, including
-                   direct, indirect, special, incidental and
-                   consequential damages, such as lost profits;
-
-                   iii) states that any provisions which differ from
-                   this Agreement are offered by that Contributor
-                   alone and not by any other party; and
-
-                   iv) states that source code for the Program is
-                   available from such Contributor, and informs
-                   licensees how to obtain it in a reasonable manner
-                   on or through a medium customarily used for
-                   software exchange.
-
-             When the Program is made available in source code form:
-                   a) it must be made available under this Agreement;
-                   and
-                   b) a copy of this Agreement must be included with
-                   each copy of the Program. 
-
-             Contributors may not remove or alter any copyright notices
-             contained within the Program.
-
-             Each Contributor must identify itself as the originator of
-             its Contribution, if any, in a manner that reasonably
-             allows subsequent Recipients to identify the originator of
-             the Contribution. 
-
-
-             4. COMMERCIAL DISTRIBUTION
-
-             Commercial distributors of software may accept certain
-             responsibilities with respect to end users, business
-             partners and the like. While this license is intended to
-             facilitate the commercial use of the Program, the
-             Contributor who includes the Program in a commercial
-             product offering should do so in a manner which does not
-             create potential liability for other Contributors.
-             Therefore, if a Contributor includes the Program in a
-             commercial product offering, such Contributor ("Commercial
-             Contributor") hereby agrees to defend and indemnify every
-             other Contributor ("Indemnified Contributor") against any
-             losses, damages and costs (collectively "Losses") arising
-             from claims, lawsuits and other legal actions brought by a
-             third party against the Indemnified Contributor to the
-             extent caused by the acts or omissions of such Commercial
-             Contributor in connection with its distribution of the
-             Program in a commercial product offering. The obligations
-             in this section do not apply to any claims or Losses
-             relating to any actual or alleged intellectual property
-             infringement. In order to qualify, an Indemnified
-             Contributor must: a) promptly notify the Commercial
-             Contributor in writing of such claim, and b) allow the
-             Commercial Contributor to control, and cooperate with the
-             Commercial Contributor in, the defense and any related
-             settlement negotiations. The Indemnified Contributor may
-             participate in any such claim at its own expense.
-
-
-             For example, a Contributor might include the Program in a
-             commercial product offering, Product X. That Contributor
-             is then a Commercial Contributor. If that Commercial
-             Contributor then makes performance claims, or offers
-             warranties related to Product X, those performance claims
-             and warranties are such Commercial Contributor's
-             responsibility alone. Under this section, the Commercial
-             Contributor would have to defend claims against the other
-             Contributors related to those performance claims and
-             warranties, and if a court requires any other Contributor
-             to pay any damages as a result, the Commercial Contributor
-             must pay those damages.
-
-
-             5. NO WARRANTY
-
-             EXCEPT AS EXPRESSLY SET FORTH IN THIS AGREEMENT, THE
-             PROGRAM IS PROVIDED ON AN "AS IS" BASIS, WITHOUT
-             WARRANTIES OR CONDITIONS OF ANY KIND, EITHER EXPRESS OR
-             IMPLIED INCLUDING, WITHOUT LIMITATION, ANY WARRANTIES OR
-             CONDITIONS OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR
-             FITNESS FOR A PARTICULAR PURPOSE. Each Recipient is solely
-             responsible for determining the appropriateness of using
-             and distributing the Program and assumes all risks
-             associated with its exercise of rights under this
-             Agreement, including but not limited to the risks and
-             costs of program errors, compliance with applicable laws,
-             damage to or loss of data, programs or equipment, and
-             unavailability or interruption of operations. 
-
-             6. DISCLAIMER OF LIABILITY
-             EXCEPT AS EXPRESSLY SET FORTH IN THIS AGREEMENT, NEITHER
-             RECIPIENT NOR ANY CONTRIBUTORS SHALL HAVE ANY LIABILITY
-             FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
-             OR CONSEQUENTIAL DAMAGES (INCLUDING WITHOUT LIMITATION
-             LOST PROFITS), HOWEVER CAUSED AND ON ANY THEORY OF
-             LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-             (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
-             OF THE USE OR DISTRIBUTION OF THE PROGRAM OR THE EXERCISE
-             OF ANY RIGHTS GRANTED HEREUNDER, EVEN IF ADVISED OF THE
-             POSSIBILITY OF SUCH DAMAGES.
-
-             7. GENERAL
-
-             If any provision of this Agreement is invalid or
-             unenforceable under applicable law, it shall not affect
-             the validity or enforceability of the remainder of the
-             terms of this Agreement, and without further action by the
-             parties hereto, such provision shall be reformed to the
-             minimum extent necessary to make such provision valid and
-             enforceable.
-
-
-             If Recipient institutes patent litigation against a
-             Contributor with respect to a patent applicable to
-             software (including a cross-claim or counterclaim in a
-             lawsuit), then any patent licenses granted by that
-             Contributor to such Recipient under this Agreement shall
-             terminate as of the date such litigation is filed. In
-             addition, If Recipient institutes patent litigation
-             against any entity (including a cross-claim or
-             counterclaim in a lawsuit) alleging that the Program
-             itself (excluding combinations of the Program with other
-             software or hardware) infringes such Recipient's
-             patent(s), then such Recipient's rights granted under
-             Section 2(b) shall terminate as of the date such
-             litigation is filed.
-
-             All Recipient's rights under this Agreement shall
-             terminate if it fails to comply with any of the material
-             terms or conditions of this Agreement and does not cure
-             such failure in a reasonable period of time after becoming
-             aware of such noncompliance. If all Recipient's rights
-             under this Agreement terminate, Recipient agrees to cease
-             use and distribution of the Program as soon as reasonably
-             practicable. However, Recipient's obligations under this
-             Agreement and any licenses granted by Recipient relating
-             to the Program shall continue and survive. 
-
-             Everyone is permitted to copy and distribute copies of
-             this Agreement, but in order to avoid inconsistency the
-             Agreement is copyrighted and may only be modified in the
-             following manner. The Agreement Steward reserves the right
-             to publish new versions (including revisions) of this
-             Agreement from time to time. No one other than the
-             Agreement Steward has the right to modify this Agreement.
-
-             IBM is the initial Agreement Steward. IBM may assign the
-             responsibility to serve as the Agreement Steward to a
-             suitable separate entity. Each new version of the
-             Agreement will be given a distinguishing version number.
-             The Program (including Contributions) may always be
-             distributed subject to the version of the Agreement under
-             which it was received. In addition, after a new version of
-             the Agreement is published, Contributor may elect to
-             distribute the Program (including its Contributions) under
-             the new version. Except as expressly stated in Sections
-             2(a) and 2(b) above, Recipient receives no rights or
-             licenses to the intellectual property of any Contributor
-             under this Agreement, whether expressly, by implication,
-             estoppel or otherwise. All rights in the Program not
-             expressly granted under this Agreement are reserved.
-
-
-             This Agreement is governed by the laws of the State of New
-             York and the intellectual property laws of the United
-             States of America. No party to this Agreement will bring a
-             legal action under this Agreement more than one year after
-             the cause of action arose. Each party waives its rights to
-             a jury trial in any resulting litigation. 
-
-
-
-*/
-
-/* (C) COPYRIGHT International Business Machines Corp. 2001,2002          */
-
+ * COPYRIGHT (c) International Business Machines Corp. 2001-2017
+ *
+ * This program is provided under the terms of the Common Public License,
+ * version 1.0 (CPL-1.0). Any use, reproduction or distribution for this
+ * software constitutes recipient's acceptance of CPL-1.0 terms which can be
+ * found in the file LICENSE file or at
+ * https://opensource.org/licenses/cpl1.0.php
+ */
 
 #include <pthread.h>
 #include <string.h>            // for memcmp() et al
@@ -299,6 +18,7 @@
 #include "host_defs.h"
 #include "h_extern.h"
 #include "tok_spec_struct.h"
+#include "trace.h"
 
 
 // forward declaration
@@ -321,7 +41,8 @@ static CK_BYTE PADDING[64] = {
 //
 //
 CK_RV
-md5_hash( SESSION         * sess,
+md5_hash( STDLL_TokData_t * tokdata,
+	  SESSION         * sess,
           CK_BBOOL          length_only,
           DIGEST_CONTEXT  * ctx,
           CK_BYTE         * in_data,
@@ -333,7 +54,7 @@ md5_hash( SESSION         * sess,
 
 
    if (!sess || !ctx || !out_data_len){
-      st_err_log(4, __FILE__, __LINE__, __FUNCTION__);
+      TRACE_ERROR("%s received bad argument(s)\n", __FUNCTION__);
       return CKR_FUNCTION_FAILED;
    }
 
@@ -342,30 +63,29 @@ md5_hash( SESSION         * sess,
       return CKR_OK;
    }
 
-   rc = md5_hash_update( sess, ctx, in_data, in_data_len );
+   rc = md5_hash_update( tokdata, sess, ctx, in_data, in_data_len );
    if (rc != CKR_OK){
-      st_err_log(4, __FILE__, __LINE__, __FUNCTION__);
-      return CKR_FUNCTION_FAILED;
+      TRACE_DEVEL("md5_hash_update failed\n");
+      return rc;
    }
-   return md5_hash_final( sess,      FALSE,
-                          ctx,
-                          out_data,  out_data_len );
+   return md5_hash_final( tokdata, sess, FALSE, ctx, out_data,  out_data_len );
 }
 
 
 //
 //
 CK_RV
-md5_hash_update( SESSION         * sess,
+md5_hash_update( STDLL_TokData_t * tokdata,
+		 SESSION         * sess,
                  DIGEST_CONTEXT  * ctx,
                  CK_BYTE         * in_data,
                  CK_ULONG          in_data_len )
 {
    if (!sess || !ctx || !in_data){
-      st_err_log(4, __FILE__, __LINE__, __FUNCTION__);
+      TRACE_ERROR("%s received bad argument(s)\n", __FUNCTION__);
       return CKR_FUNCTION_FAILED;
    }
-   return ckm_md5_update( (MD5_CONTEXT *)ctx->context,
+   return ckm_md5_update( tokdata, (MD5_CONTEXT *)ctx->context,
                           in_data, in_data_len );
 }
 
@@ -373,7 +93,8 @@ md5_hash_update( SESSION         * sess,
 //
 //
 CK_RV
-md5_hash_final( SESSION         * sess,
+md5_hash_final( STDLL_TokData_t * tokdata,
+		SESSION         * sess,
                 CK_BYTE           length_only,
                 DIGEST_CONTEXT  * ctx,
                 CK_BYTE         * out_data,
@@ -383,7 +104,7 @@ md5_hash_final( SESSION         * sess,
 
 
    if (!sess || !ctx || !out_data_len){
-      st_err_log(4, __FILE__, __LINE__, __FUNCTION__);
+      TRACE_ERROR("%s received bad argument(s)\n", __FUNCTION__);
       return CKR_FUNCTION_FAILED;
    }
    if (length_only == TRUE) {
@@ -391,7 +112,7 @@ md5_hash_final( SESSION         * sess,
       return CKR_OK;
    }
 
-   rc = ckm_md5_final( (MD5_CONTEXT *)ctx->context,
+   rc = ckm_md5_final( tokdata, (MD5_CONTEXT *)ctx->context,
                        out_data, MD5_HASH_SIZE );
 
    if (rc == CKR_OK) {
@@ -408,7 +129,8 @@ md5_hash_final( SESSION         * sess,
 //    CKM_MD5_HMAC_GENERAL
 //
 CK_RV
-md5_hmac_sign( SESSION              * sess,
+md5_hmac_sign( STDLL_TokData_t      * tokdata,
+	       SESSION              * sess,
                CK_BBOOL               length_only,
                SIGN_VERIFY_CONTEXT  * ctx,
                CK_BYTE              * in_data,
@@ -429,7 +151,7 @@ md5_hmac_sign( SESSION              * sess,
 
 
    if (!sess || !ctx || !out_data_len){
-      st_err_log(4, __FILE__, __LINE__, __FUNCTION__);
+      TRACE_ERROR("%s received bad argument(s)\n", __FUNCTION__);
       return CKR_FUNCTION_FAILED;
    }
 
@@ -452,13 +174,13 @@ md5_hmac_sign( SESSION              * sess,
 
    memset( &digest_ctx, 0x0, sizeof(DIGEST_CONTEXT) );
 
-   rc = object_mgr_find_in_map1( ctx->key, &key_obj );
+   rc = object_mgr_find_in_map1( tokdata, ctx->key, &key_obj );
    if (rc != CKR_OK)
       return rc;
 
    rc = template_attribute_find( key_obj->template, CKA_VALUE, &attr );
    if (rc == FALSE){
-      st_err_log(4, __FILE__, __LINE__, __FUNCTION__);
+      TRACE_ERROR("Could not find CKA_VALUE in the template\n");
       return CKR_FUNCTION_FAILED;
    }
    else
@@ -472,23 +194,20 @@ md5_hmac_sign( SESSION              * sess,
       digest_mech.ulParameterLen = 0;
       digest_mech.pParameter     = NULL;
 
-      rc = digest_mgr_init( sess, &digest_ctx, &digest_mech );
+      rc = digest_mgr_init( tokdata, sess, &digest_ctx, &digest_mech );
       if (rc != CKR_OK)
       {
-         digest_mgr_cleanup( &digest_ctx );
          return rc;
       }
 
       hash_len = sizeof(hash);
-      rc = digest_mgr_digest( sess, FALSE, &digest_ctx,
+      rc = digest_mgr_digest( tokdata, sess, FALSE, &digest_ctx,
                               attr->pValue, attr->ulValueLen,
                               hash,  &hash_len );
       if (rc != CKR_OK) {
-         digest_mgr_cleanup( &digest_ctx );
          return rc;
       }
 
-      digest_mgr_cleanup( &digest_ctx );
       memset( &digest_ctx, 0x0, sizeof(DIGEST_CONTEXT) );
 
       for (i=0; i < hash_len; i++) {
@@ -518,82 +237,77 @@ md5_hmac_sign( SESSION              * sess,
 
    // inner hash
    //
-   rc = digest_mgr_init( sess, &digest_ctx, &digest_mech );
+   rc = digest_mgr_init( tokdata, sess, &digest_ctx, &digest_mech );
    if (rc != CKR_OK)
    {
-      st_err_log(123, __FILE__, __LINE__);
-      digest_mgr_cleanup( &digest_ctx );
+      TRACE_DEVEL("Digest Mgr Init failed.\n");
       return rc;
    }
 
-   rc = digest_mgr_digest_update( sess, &digest_ctx, k_ipad, MD5_BLOCK_SIZE );
+   rc = digest_mgr_digest_update( tokdata, sess, &digest_ctx, k_ipad,
+				  MD5_BLOCK_SIZE );
    if (rc != CKR_OK)
    {
-      st_err_log(123, __FILE__, __LINE__);
-      digest_mgr_cleanup( &digest_ctx );
+      TRACE_DEVEL("Digest Mgr Update failed.\n");
       return rc;
    }
 
-   rc = digest_mgr_digest_update( sess, &digest_ctx, in_data, in_data_len );
+   rc = digest_mgr_digest_update( tokdata, sess, &digest_ctx, in_data,
+				  in_data_len );
    if (rc != CKR_OK)
    {
-      st_err_log(123, __FILE__, __LINE__);
-      digest_mgr_cleanup( &digest_ctx );
+      TRACE_DEVEL("Digest Mgr Update failed.\n");
       return rc;
    }
 
    hash_len = sizeof(hash);
-   rc = digest_mgr_digest_final( sess, FALSE, &digest_ctx, hash, &hash_len );
+   rc = digest_mgr_digest_final( tokdata, sess, FALSE, &digest_ctx, hash,
+				 &hash_len );
    if (rc != CKR_OK)
    {
-      st_err_log(126, __FILE__, __LINE__);
-      digest_mgr_cleanup( &digest_ctx );
+      TRACE_DEVEL("Digest Mgr Final failed.\n");
       return rc;
    }
 
-   digest_mgr_cleanup( &digest_ctx );
    memset( &digest_ctx, 0x0, sizeof(DIGEST_CONTEXT) );
 
 
    // outer hash
    //
-   rc = digest_mgr_init( sess, &digest_ctx, &digest_mech );
+   rc = digest_mgr_init( tokdata, sess, &digest_ctx, &digest_mech );
    if (rc != CKR_OK)
    {
-      st_err_log(123, __FILE__, __LINE__);
-      digest_mgr_cleanup( &digest_ctx );
+      TRACE_DEVEL("Digest Mgr Init failed.\n");
       return rc;
    }
 
-   rc = digest_mgr_digest_update( sess, &digest_ctx, k_opad, MD5_BLOCK_SIZE );
+   rc = digest_mgr_digest_update( tokdata, sess, &digest_ctx, k_opad,
+				  MD5_BLOCK_SIZE );
    if (rc != CKR_OK)
    {
-      st_err_log(123, __FILE__, __LINE__);
-      digest_mgr_cleanup( &digest_ctx );
+      TRACE_DEVEL("Digest Mgr Update failed.\n");
       return rc;
    }
 
-   rc = digest_mgr_digest_update( sess, &digest_ctx, hash, hash_len );
+   rc = digest_mgr_digest_update( tokdata, sess, &digest_ctx, hash, hash_len );
    if (rc != CKR_OK)
    {
-      st_err_log(123, __FILE__, __LINE__);
-      digest_mgr_cleanup( &digest_ctx );
+      TRACE_DEVEL("Digest Mgr Update failed.\n");
       return rc;
    }
 
    hash_len = sizeof(hash);
-   rc = digest_mgr_digest_final( sess, FALSE, &digest_ctx, hash, &hash_len );
+   rc = digest_mgr_digest_final( tokdata, sess, FALSE, &digest_ctx, hash,
+				 &hash_len );
    if (rc != CKR_OK)
    {
-      st_err_log(126, __FILE__, __LINE__);
-      digest_mgr_cleanup( &digest_ctx );
+      TRACE_DEVEL("Digest Mgr Final failed.\n");
       return rc;
    }
 
    memcpy( out_data, hash, hmac_len );
    *out_data_len = hmac_len;
 
-   digest_mgr_cleanup( &digest_ctx );
    return CKR_OK;
 }
 
@@ -601,7 +315,8 @@ md5_hmac_sign( SESSION              * sess,
 //
 //
 CK_RV
-md5_hmac_verify( SESSION              * sess,
+md5_hmac_verify( STDLL_TokData_t      * tokdata,
+		 SESSION              * sess,
                  SIGN_VERIFY_CONTEXT  * ctx,
                  CK_BYTE              * in_data,
                  CK_ULONG               in_data_len,
@@ -614,7 +329,7 @@ md5_hmac_verify( SESSION              * sess,
    CK_RV                rc;
 
    if (!sess || !ctx || !in_data || !signature){
-      st_err_log(4, __FILE__, __LINE__, __FUNCTION__);
+      TRACE_ERROR("%s received bad argument(s)\n", __FUNCTION__);
       return CKR_FUNCTION_FAILED;
    }
    if (ctx->mech.mechanism == CKM_MD5_HMAC_GENERAL)
@@ -624,14 +339,14 @@ md5_hmac_verify( SESSION              * sess,
 
    memset( &hmac_ctx, 0, sizeof(SIGN_VERIFY_CONTEXT) );
 
-   rc = sign_mgr_init( sess, &hmac_ctx, &ctx->mech, FALSE, ctx->key );
+   rc = sign_mgr_init( tokdata, sess, &hmac_ctx, &ctx->mech, FALSE, ctx->key );
    if (rc != CKR_OK) {
       sign_mgr_cleanup( &hmac_ctx );
       return rc;
    }
 
    len = sizeof(hmac);
-   rc = sign_mgr_sign( sess, FALSE, &hmac_ctx,
+   rc = sign_mgr_sign( tokdata, sess, FALSE, &hmac_ctx,
                        in_data, in_data_len,
                        hmac,   &len );
    if (rc != CKR_OK) {
@@ -640,11 +355,11 @@ md5_hmac_verify( SESSION              * sess,
    }
 
    if ((len != hmac_len) || (len != sig_len)){
-      st_err_log(46, __FILE__, __LINE__);
+      TRACE_ERROR("%s\n", ock_err(ERR_SIGNATURE_LEN_RANGE));
       return CKR_SIGNATURE_LEN_RANGE;
    }
    if (memcmp(hmac, signature, hmac_len) != 0){
-      st_err_log(47, __FILE__, __LINE__);
+      TRACE_ERROR("%s\n", ock_err(ERR_SIGNATURE_INVALID));
       rc = CKR_SIGNATURE_INVALID;
    }
    sign_mgr_cleanup( &hmac_ctx );
@@ -658,7 +373,7 @@ md5_hmac_verify( SESSION              * sess,
 
 
 void
-ckm_md5_init( MD5_CONTEXT *context )
+ckm_md5_init( STDLL_TokData_t *tokdata, MD5_CONTEXT *context )
 {
   context->i[0] = context->i[1] = 0;
 
@@ -674,7 +389,8 @@ ckm_md5_init( MD5_CONTEXT *context )
 //
 //
 CK_RV
-ckm_md5_update( MD5_CONTEXT  * context,
+ckm_md5_update( STDLL_TokData_t * tokdata,
+		MD5_CONTEXT  * context,
                 CK_BYTE      * in_data,
                 CK_ULONG       in_data_len )
 {
@@ -707,7 +423,7 @@ ckm_md5_update( MD5_CONTEXT  * context,
                 (((CK_ULONG)context->in[ii+2]) << 16) |
                 (((CK_ULONG)context->in[ii+1]) << 8) |
                  ((CK_ULONG)context->in[ii]);
-      ckm_md5_transform (context->buf, in);
+      ckm_md5_transform (tokdata, context->buf, in);
       mdi = 0;
     }
   }
@@ -719,7 +435,8 @@ ckm_md5_update( MD5_CONTEXT  * context,
 //
 //
 CK_RV
-ckm_md5_final( MD5_CONTEXT *context,
+ckm_md5_final( STDLL_TokData_t *tokdata,
+	       MD5_CONTEXT *context,
                CK_BYTE     *out_data,
                CK_ULONG     out_data_len )
 {
@@ -728,9 +445,14 @@ ckm_md5_final( MD5_CONTEXT *context,
   CK_ULONG  i, ii;
   CK_ULONG  padLen;
 
-   if (!out_data || (out_data_len < MD5_HASH_SIZE)){
-      st_err_log(4, __FILE__, __LINE__, __FUNCTION__);
-      return CKR_FUNCTION_FAILED;
+   if (!out_data) {
+	TRACE_ERROR("%s received bad argument(s)\n", __FUNCTION__);
+	return CKR_FUNCTION_FAILED;
+   }
+
+   if (out_data_len < MD5_HASH_SIZE) {
+      TRACE_ERROR("%s\n", ock_err(ERR_BUFFER_TOO_SMALL));
+      return CKR_BUFFER_TOO_SMALL;
    }
   // save number of bits
   //
@@ -744,7 +466,7 @@ ckm_md5_final( MD5_CONTEXT *context,
   // pad out to 56 mod 64
   //
   padLen = (mdi < 56) ? (56 - mdi) : (120 - mdi);
-  ckm_md5_update( context, PADDING, padLen );
+  ckm_md5_update( tokdata, context, PADDING, padLen );
 
   // append length in bits and transform
   //
@@ -753,7 +475,7 @@ ckm_md5_final( MD5_CONTEXT *context,
             (((CK_ULONG)context->in[ii+2]) << 16) |
             (((CK_ULONG)context->in[ii+1]) << 8) |
             ((CK_ULONG)context->in[ii]);
-  ckm_md5_transform (context->buf, in);
+  ckm_md5_transform (tokdata, context->buf, in);
 
   // store buffer in digest
   //
@@ -899,7 +621,7 @@ static CK_ULONG T[64] = { 0xD76AA478,  /* T[01]                               */
 // Basic MD5 step. Transform buf based on in.
 //
 void
-ckm_md5_transform( CK_ULONG *long_buf, CK_ULONG *long_in )
+ckm_md5_transform(STDLL_TokData_t *tokdata, CK_ULONG *long_buf, CK_ULONG *long_in)
 {
 
    /*-------------------------------------------------------------------------*/
@@ -935,7 +657,7 @@ ckm_md5_transform( CK_ULONG *long_buf, CK_ULONG *long_in )
    CK_ULONG CC = 0x00000000;           /* Temp. save areas for A, B, C, D     */
    CK_ULONG DD = 0x00000000;           /* Temp. save areas for A, B, C, D     */
 
-   CK_ULONG_32 buf[4];                 // temps for long_buf[i]                     
+   CK_ULONG_32 buf[4];                 // temps for long_buf[i]
    CK_ULONG_32 in[16];                 // temps for long_in[i]
 
    int i;                              // loop counter
@@ -957,8 +679,8 @@ ckm_md5_transform( CK_ULONG *long_buf, CK_ULONG *long_in )
    for (i=0;i<4;i++) {
      buf[i] = (CK_ULONG_32)long_buf[i];
      in[i] = (CK_ULONG_32)long_in[i];
-   } 
-   for (i=4;i<16;i++) 
+   }
+   for (i=4;i<16;i++)
      in[i] = (CK_ULONG_32)long_in[i];
 
 
@@ -1091,4 +813,3 @@ ckm_md5_transform( CK_ULONG *long_buf, CK_ULONG *long_in )
    /* Undefine the ROUND_FCN macro we used in this function.                  */
    #undef ROUND_FCN
 }
-
