@@ -279,22 +279,20 @@ CK_RV _UnlockMutex(MUTEX * mutex)
 
 }
 
-static int spinxplfd = -1;
-
-CK_RV CreateXProcLock(void)
+CK_RV CreateXProcLock(char *tokname, STDLL_TokData_t *tokdata)
 {
-	CK_BYTE lockfile[PATH_MAX];
-	CK_BYTE lockdir[PATH_MAX];
+	CK_BYTE lockfile[2*PATH_MAX + sizeof(LOCKDIR_PATH) + 6];
+	CK_BYTE lockdir[PATH_MAX + sizeof(LOCKDIR_PATH)];
 	struct group *grp;
 	struct stat statbuf;
 	mode_t mode = (S_IRUSR | S_IRGRP);
 	int ret = -1;
 
-	if (spinxplfd == -1) {
+	if (tokdata->spinxplfd == -1) {
 
 		if (token_specific.t_creatlock != NULL) {
-			spinxplfd = token_specific.t_creatlock();
-			if (spinxplfd != -1)
+			tokdata->spinxplfd = token_specific.t_creatlock();
+			if (tokdata->spinxplfd != -1)
 				return CKR_OK;
 			else
 				return CKR_FUNCTION_FAILED;
@@ -302,7 +300,10 @@ CK_RV CreateXProcLock(void)
 
 		/** create lock subdir for each token if it doesn't exist.
 		  * The root directory should be created in slotmgr daemon **/
-		sprintf(lockdir, "%s/%s", LOCKDIR_PATH, SUB_DIR);
+		if (strlen(tokname) > 0)
+			sprintf(lockdir, "%s/%s", LOCKDIR_PATH, tokname);
+		else
+			sprintf(lockdir, "%s/%s", LOCKDIR_PATH, SUB_DIR);
 
 		ret = stat(lockdir, &statbuf);
 		if (ret != 0 && errno == ENOENT) {
@@ -339,16 +340,20 @@ CK_RV CreateXProcLock(void)
 		}
 
 		/* create user lock file */
-		sprintf(lockfile, "%s/%s/LCK..%s",
+		if (strlen(tokname) > 0)
+			sprintf(lockfile, "%s/%s/LCK..%s",
+				LOCKDIR_PATH, tokname, tokname);
+		else
+			sprintf(lockfile, "%s/%s/LCK..%s",
 				LOCKDIR_PATH, SUB_DIR, SUB_DIR);
 
 		if (stat(lockfile, &statbuf) == 0)
-			spinxplfd = open(lockfile, O_RDONLY, mode);
+			tokdata->spinxplfd = open(lockfile, O_RDONLY, mode);
 		else {
-			spinxplfd = open(lockfile, O_CREAT | O_RDONLY, mode);
-			if (spinxplfd != -1) {
+			tokdata->spinxplfd = open(lockfile, O_CREAT | O_RDONLY, mode);
+			if (tokdata->spinxplfd != -1) {
 				/* umask may prevent correct mode,so set it. */
-				if (fchmod(spinxplfd, mode) == -1) {
+				if (fchmod(tokdata->spinxplfd, mode) == -1) {
 					OCK_SYSLOG(LOG_ERR, "fchmod(%s): %s\n",
 							lockfile, strerror(errno));
 					goto err;
@@ -356,8 +361,8 @@ CK_RV CreateXProcLock(void)
 
 				grp = getgrnam("pkcs11");
 				if (grp != NULL) {
-					if (fchown(spinxplfd, -1, grp->gr_gid)
-							== -1) {
+					if (fchown(tokdata->spinxplfd, -1,
+						   grp->gr_gid) == -1) {
 						OCK_SYSLOG(LOG_ERR,
 								"fchown(%s): %s\n",
 								lockfile,
@@ -371,7 +376,7 @@ CK_RV CreateXProcLock(void)
 				}
 			}
 		}
-		if (spinxplfd == -1) {
+		if (tokdata->spinxplfd == -1) {
 			OCK_SYSLOG(LOG_ERR, "open(%s): %s\n",
 					lockfile, strerror(errno));
 			return CKR_FUNCTION_FAILED;
@@ -381,40 +386,44 @@ CK_RV CreateXProcLock(void)
 	return CKR_OK;
 
 err:
-	if (spinxplfd != -1)
-		close(spinxplfd);
+	if (tokdata->spinxplfd != -1)
+		close(tokdata->spinxplfd);
 	return CKR_FUNCTION_FAILED;
 }
 
-void CloseXProcLock(void)
+void CloseXProcLock(STDLL_TokData_t *tokdata)
 {
-	if (spinxplfd != -1)
-		close(spinxplfd);
+	if (tokdata->spinxplfd != -1)
+		close(tokdata->spinxplfd);
 }
 
-CK_RV XProcLock(void)
+CK_RV XProcLock(STDLL_TokData_t *tokdata)
 {
-	if (spinxplfd != -1)
-		flock(spinxplfd, LOCK_EX);
-	else
+	if (tokdata->spinxplfd != -1)
+		flock(tokdata->spinxplfd, LOCK_EX);
+	else {
 		TRACE_DEVEL("No file descriptor to lock with.\n");
+		return CKR_CANT_LOCK;
+	}
 
 	return CKR_OK;
 }
 
-CK_RV XProcUnLock(void)
+CK_RV XProcUnLock(STDLL_TokData_t *tokdata)
 {
-	if (spinxplfd != -1)
-		flock(spinxplfd, LOCK_UN);
-	else
+	if (tokdata->spinxplfd != -1)
+		flock(tokdata->spinxplfd, LOCK_UN);
+	else {
 		TRACE_DEVEL("No file descriptor to unlock with.\n");
+		return CKR_CANT_LOCK;
+	}
 
 	return CKR_OK;
 }
 
-void XProcLock_Init(void)
+void XProcLock_Init(STDLL_TokData_t *tokdata)
 {
-	spinxplfd = -1;
+	tokdata->spinxplfd = -1;
 }
 
 //
@@ -427,37 +436,35 @@ extern CK_CHAR label[];
 
 //
 //
-void init_slotInfo(void)
+void init_slotInfo(CK_SLOT_INFO *slot_info)
 {
-	memset(&slot_info.slotDescription, ' ',
-	       sizeof(slot_info.slotDescription));
-	memset(&slot_info.manufacturerID, ' ',
-	       sizeof(slot_info.manufacturerID));
+	memset(slot_info->slotDescription, ' ',
+	       sizeof(slot_info->slotDescription));
+	memset(slot_info->manufacturerID, ' ',
+	       sizeof(slot_info->manufacturerID));
 
-	memcpy(&slot_info.slotDescription, descr, strlen((char *)descr));
-	memcpy(&slot_info.manufacturerID, manuf, strlen((char *)manuf));
+	memcpy(slot_info->slotDescription, descr, strlen((char *)descr));
+	memcpy(slot_info->manufacturerID, manuf, strlen((char *)manuf));
 
-	slot_info.hardwareVersion.major = 1;
-	slot_info.hardwareVersion.minor = 0;
-	slot_info.firmwareVersion.major = 1;
-	slot_info.firmwareVersion.minor = 0;
-	slot_info.flags = CKF_TOKEN_PRESENT | CKF_HW_SLOT;
+	slot_info->hardwareVersion.major = 1;
+	slot_info->hardwareVersion.minor = 0;
+	slot_info->firmwareVersion.major = 1;
+	slot_info->firmwareVersion.minor = 0;
+	slot_info->flags = CKF_TOKEN_PRESENT | CKF_HW_SLOT;
 }
 
 //
 //
-void init_tokenInfo(void)
+void init_tokenInfo(TOKEN_DATA *nv_token_data)
 {
-	CK_TOKEN_INFO_32 *token_info = NULL;
-
-	token_info = &nv_token_data->token_info;
+	CK_TOKEN_INFO_32 *token_info = &nv_token_data->token_info;
 
 	memset(token_info->manufacturerID, ' ',
 	       sizeof(token_info->manufacturerID));
 	memset(token_info->model, ' ', sizeof(token_info->model));
 	memset(token_info->serialNumber, ' ', sizeof(token_info->serialNumber));
 
-	memcpy(token_info->label, nv_token_data->token_info.label, 32);
+	memcpy(token_info->label, label, strlen((char *)label));
 
 	memcpy(token_info->manufacturerID, manuf, strlen((char *)manuf));
 	memcpy(token_info->model, model, strlen((char *)model));
@@ -509,58 +516,58 @@ void init_tokenInfo(void)
 
 //
 //
-CK_RV init_token_data(CK_SLOT_ID slot_id)
+CK_RV init_token_data(STDLL_TokData_t *tokdata, CK_SLOT_ID slot_id)
 {
 	CK_RV rc;
 
-	memset((char *)nv_token_data, 0, sizeof(nv_token_data));
+	memset((char *)tokdata->nv_token_data, 0, sizeof(TOKEN_DATA));
 
 	// the normal USER pin is not set when the token is initialized
 	//
-	memcpy(nv_token_data->user_pin_sha, "00000000000000000000",
+	memcpy(tokdata->nv_token_data->user_pin_sha, "00000000000000000000",
 	       SHA1_HASH_SIZE);
-	memcpy(nv_token_data->so_pin_sha, default_so_pin_sha, SHA1_HASH_SIZE);
+	memcpy(tokdata->nv_token_data->so_pin_sha, default_so_pin_sha, SHA1_HASH_SIZE);
 
-	memset(user_pin_md5, 0x0, MD5_HASH_SIZE);
-	memcpy(so_pin_md5, default_so_pin_md5, MD5_HASH_SIZE);
+	memset(tokdata->user_pin_md5, 0x0, MD5_HASH_SIZE);
+	memcpy(tokdata->so_pin_md5, default_so_pin_md5, MD5_HASH_SIZE);
 
-	memcpy(nv_token_data->next_token_object_name, "00000000", 8);
+	memcpy(tokdata->nv_token_data->next_token_object_name, "00000000", 8);
 
 	// generate the master key used for signing the Operation State information
 	//                          `
-	memset(nv_token_data->token_info.label, ' ',
-	       sizeof(nv_token_data->token_info.label));
-	memcpy(nv_token_data->token_info.label, label, strlen((char *)label));
+	memset(tokdata->nv_token_data->token_info.label, ' ',
+	       sizeof(tokdata->nv_token_data->token_info.label));
+	memcpy(tokdata->nv_token_data->token_info.label, label, strlen((char *)label));
 
-	nv_token_data->tweak_vector.allow_weak_des = TRUE;
-	nv_token_data->tweak_vector.check_des_parity = FALSE;
-	nv_token_data->tweak_vector.allow_key_mods = TRUE;
-	nv_token_data->tweak_vector.netscape_mods = TRUE;
+	tokdata->nv_token_data->tweak_vector.allow_weak_des = TRUE;
+	tokdata->nv_token_data->tweak_vector.check_des_parity = FALSE;
+	tokdata->nv_token_data->tweak_vector.allow_key_mods = TRUE;
+	tokdata->nv_token_data->tweak_vector.netscape_mods = TRUE;
 
-	init_tokenInfo();
+	init_tokenInfo(tokdata->nv_token_data);
 
 	if (token_specific.t_init_token_data) {
-		rc = token_specific.t_init_token_data(slot_id);
+		rc = token_specific.t_init_token_data(tokdata, slot_id);
 		if (rc != CKR_OK)
 			return rc;
 	} else {
 		//
 		// FIXME: erase the token object index file (and all token objects)
 		//
-		rc = generate_master_key(master_key);
+		rc = generate_master_key(tokdata, tokdata->master_key);
 		if (rc != CKR_OK) {
 			TRACE_DEVEL("generate_master_key failed.\n");
 			return CKR_FUNCTION_FAILED;
 		}
 
-		rc = save_masterkey_so();
+		rc = save_masterkey_so(tokdata);
 		if (rc != CKR_OK) {
 			TRACE_DEVEL("save_masterkey_so failed.\n");
 			return rc;
 		}
 	}
 
-	rc = save_token_data(slot_id);
+	rc = save_token_data(tokdata, slot_id);
 
 	return rc;
 }
@@ -737,16 +744,17 @@ CK_RV parity_is_odd(CK_BYTE b)
 		return FALSE;
 }
 
-CK_RV attach_shm(CK_SLOT_ID slot_id, LW_SHM_TYPE **shm)
+CK_RV attach_shm(STDLL_TokData_t *tokdata, CK_SLOT_ID slot_id)
 {
 	CK_RV rc = CKR_OK;
 	int ret;
 	char buf[PATH_MAX];
+	LW_SHM_TYPE **shm = &tokdata->global_shm;
 
 	if (token_specific.t_attach_shm != NULL)
-		return token_specific.t_attach_shm(slot_id, shm);
+		return token_specific.t_attach_shm(tokdata, slot_id);
 
-	XProcLock();
+	XProcLock(tokdata);
 
 	/*
 	 * Attach to an existing shared memory region or create it if it doesn't
@@ -761,22 +769,22 @@ CK_RV attach_shm(CK_SLOT_ID slot_id, LW_SHM_TYPE **shm)
 	}
 
 done:
-	XProcUnLock();
+	XProcUnLock(tokdata);
 	return rc;
 }
 
-CK_RV detach_shm()
+CK_RV detach_shm(STDLL_TokData_t *tokdata)
 {
 	CK_RV rc = CKR_OK;
 
-	XProcLock();
+	XProcLock(tokdata);
 
-	if (sm_close((void *)global_shm, 0)) {
+	if (sm_close((void *)tokdata->global_shm, 0)) {
 		TRACE_DEVEL("sm_close failed.\n");
 		rc = CKR_FUNCTION_FAILED;
 	}
 
-	XProcUnLock();
+	XProcUnLock(tokdata);
 
 	return rc;
 }
@@ -803,7 +811,8 @@ CK_RV get_sha_size(CK_ULONG mech, CK_ULONG *hsize)
 }
 
 /* Compute specified SHA using either software or token implementation */
-CK_RV compute_sha(CK_BYTE * data, CK_ULONG len, CK_BYTE *hash, CK_ULONG mech)
+CK_RV compute_sha(STDLL_TokData_t *tokdata, CK_BYTE * data, CK_ULONG len,
+		  CK_BYTE *hash, CK_ULONG mech)
 {
 	DIGEST_CONTEXT ctx;
 	CK_ULONG hash_len;
@@ -816,16 +825,17 @@ CK_RV compute_sha(CK_BYTE * data, CK_ULONG len, CK_BYTE *hash, CK_ULONG mech)
 	if (rv != CKR_OK)
 		return rv;
 
-	rv = sha_init(NULL, &ctx, &ctx.mech);
+	rv = sha_init(tokdata, NULL, &ctx, &ctx.mech);
 	if (rv != CKR_OK) {
 		TRACE_DEBUG("failed to create digest.\n");
 		return rv;
 	}
-	return sha_hash(NULL, FALSE, &ctx, data, len, hash, &hash_len);
+	return sha_hash(tokdata, NULL, FALSE, &ctx, data, len, hash, &hash_len);
 }
 
 /* Compute SHA1 using software implementation */
-CK_RV compute_sha1(CK_BYTE * data, CK_ULONG len, CK_BYTE *hash)
+CK_RV compute_sha1(STDLL_TokData_t *tokdata, CK_BYTE * data, CK_ULONG len,
+		   CK_BYTE *hash)
 {
 	// XXX KEY
 	DIGEST_CONTEXT ctx;
@@ -840,26 +850,28 @@ CK_RV compute_sha1(CK_BYTE * data, CK_ULONG len, CK_BYTE *hash)
 	return sw_sha1_hash(&ctx, data, len, hash, &hash_len);
 }
 
-CK_RV compute_md5(CK_BYTE * data, CK_ULONG len, CK_BYTE * hash)
+CK_RV compute_md5(STDLL_TokData_t  *tokdata, CK_BYTE * data, CK_ULONG len,
+		  CK_BYTE * hash)
 {
 	MD5_CONTEXT ctx;
 
 	memset(&ctx, 0x0, sizeof(ctx));
 
-	ckm_md5_init(&ctx);
-	ckm_md5_update(&ctx, data, len);
-	ckm_md5_final(&ctx, hash, MD5_HASH_SIZE);
+	ckm_md5_init(tokdata, &ctx);
+	ckm_md5_update(tokdata, &ctx, data, len);
+	ckm_md5_final(tokdata, &ctx, hash, MD5_HASH_SIZE);
 
 	return CKR_OK;
 }
 
-CK_RV get_keytype(CK_OBJECT_HANDLE hkey, CK_KEY_TYPE *keytype)
+CK_RV get_keytype(STDLL_TokData_t *tokdata, CK_OBJECT_HANDLE hkey,
+		  CK_KEY_TYPE *keytype)
 {
 	CK_RV rc;
 	OBJECT *key_obj = NULL;
 	CK_ATTRIBUTE *attr = NULL;
 
-	rc = object_mgr_find_in_map1(hkey, &key_obj);
+	rc = object_mgr_find_in_map1(tokdata, hkey, &key_obj);
 	if (rc != CKR_OK) {
 		TRACE_DEVEL("object_mgr_find_in_map1 failed.\n");
 		return rc;
