@@ -15,17 +15,26 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <grp.h>
+#include <string.h>
+#include <openssl/evp.h>
 
 #include "log.h"
 #include "slotmgr.h"
 #include "pkcsslotd.h"
 #include "parser.h"
 
+#define OBJ_DIR "TOK_OBJ"
+#define MD5_HASH_SIZE	16
+
+typedef char md5_hash_entry[MD5_HASH_SIZE];
+md5_hash_entry tokname_hash_table[NUMBER_SLOTS_MANAGED];
+
 Slot_Mgr_Shr_t	*shmp;     // pointer to the shared memory region.
 int		shmid;
 key_t		tok;
 Slot_Info_t_64  sinfo[NUMBER_SLOTS_MANAGED];
-unsigned char NumberSlotsInDB = 0;
+Slot_Info_t_64  *psinfo;
+unsigned int NumberSlotsInDB = 0;
 
 int socketfd;
 Slot_Mgr_Socket_t      socketData;
@@ -58,6 +67,48 @@ DumpSharedMemory(void)
 	return;
 }
 
+int compute_hash(int hash_type, int buf_size, char *buf, char *digest)
+{
+	EVP_MD_CTX *md_ctx = NULL;
+	unsigned int result_size;
+	int rc;
+
+	md_ctx = EVP_MD_CTX_create();
+
+	switch (hash_type) {
+	case HASH_SHA1:
+		rc = EVP_DigestInit(md_ctx, EVP_sha1());
+		break;
+	case HASH_MD5:
+		rc = EVP_DigestInit(md_ctx, EVP_md5());
+		break;
+	default:
+		EVP_MD_CTX_destroy(md_ctx);
+		return -1;
+	break;
+	}
+
+	if (rc != 1) {
+		fprintf(stderr, "EVP_DigestInit() failed: rc = %d\n", rc);
+		return -1;
+	}
+
+        rc = EVP_DigestUpdate(md_ctx, buf, buf_size);
+        if (rc != 1) {
+		fprintf(stderr, "EVP_DigestUpdate() failed: rc = %d\n", rc);
+		return -1;
+        }
+
+	result_size = EVP_MD_CTX_size(md_ctx);
+	rc = EVP_DigestFinal(md_ctx, (unsigned char *)digest, &result_size);
+        if (rc != 1) {
+		fprintf(stderr, "EVP_DigestFinal() failed: rc = %d\n", rc);
+		return -1;
+        }
+	EVP_MD_CTX_destroy(md_ctx);
+        return 0;
+}
+
 /** This function does basic sanity checks to make sure the
  *  eco system is in place for opencryptoki to run properly.
  **/
@@ -135,6 +186,109 @@ void run_sanity_checks()
 	}
 }
 
+int is_duplicate(md5_hash_entry hash, md5_hash_entry *hash_table) {
+	int i;
+
+	for(i = 0; i < NUMBER_SLOTS_MANAGED; i++) {
+		if (memcmp(hash_table[i], hash, sizeof(md5_hash_entry)) == 0)
+			return 1;
+	}
+	return 0;
+}
+
+int chk_create_tokdir(Slot_Info_t_64* psinfo) {
+	struct stat sbuf;
+	char tokendir[PATH_MAX];
+	struct group *grp;
+	gid_t grpid;
+	int uid, rc;
+	mode_t proc_umask;
+	char *tokdir = psinfo->tokname;
+	unsigned char token_md5_hash[MD5_HASH_SIZE];
+
+	/* skip if no dedicated token directory is required */
+	if (!tokdir || strlen(tokdir) == 0)
+		return 0;
+
+	proc_umask = umask(0);
+
+	/* get 'PKCS11' group id */
+	uid = (int) geteuid();
+	grp = getgrnam("pkcs11");
+	if (!grp) {
+		fprintf(stderr, "PKCS11 group does not exist [errno=%d].\n", errno);
+		return errno;
+	} else
+		grpid = grp->gr_gid;
+
+	/* calculate md5 hash from token name */
+	rc = compute_md5(tokdir, strlen(tokdir), token_md5_hash);
+	if (rc) {
+		fprintf(stderr, "Error calculating MD5 of token name!\n");
+		return -1;
+	}
+	/* check for duplicate token names */
+	if (is_duplicate(token_md5_hash, tokname_hash_table)) {
+		fprintf(stderr, "Duplicate token name '%s'!\n", tokdir);
+		return -1;
+	}
+
+	/* add entry into hash table */
+	memcpy(tokname_hash_table[psinfo->slot_number], token_md5_hash,
+	       MD5_HASH_SIZE);
+
+	/* Create token specific directory */
+	sprintf(tokendir, "%s/%s", CONFIG_PATH, tokdir);
+	rc = stat(tokendir, &sbuf);
+	if (rc != 0 && errno == ENOENT) {
+		/* directory does not exist, create it */
+		rc = mkdir(tokendir, S_IRWXU | S_IRWXG);
+		if (rc != 0) {
+			fprintf(stderr,
+				"Creating directory '%s' failed [errno=%d].\n",
+				tokendir, errno);
+			umask(proc_umask);
+			return rc;
+		}
+
+		rc = chown(tokendir, uid, grpid);
+		if (rc != 0) {
+			fprintf(stderr,
+				"Could not set PKCS11 group permission [errno=%d].\n",
+				errno);
+			umask(proc_umask);
+			return rc;
+		}
+
+	}
+
+	/* Create TOK_OBJ directory */
+	sprintf(tokendir, "%s/%s/%s", CONFIG_PATH, tokdir, OBJ_DIR);
+	rc = stat(tokendir, &sbuf);
+	if (rc != 0 && errno == ENOENT) {
+		/* directory does not exist, create it */
+		rc = mkdir(tokendir, S_IRWXU | S_IRWXG);
+		if (rc != 0) {
+			fprintf(stderr,
+				"Creating directory '%s' failed [errno=%d].\n",
+				tokendir, errno);
+			umask(proc_umask);
+			return rc;
+		}
+
+		rc = chown(tokendir, uid, grpid);
+		if (rc != 0) {
+			fprintf(stderr,
+				"Could not set PKCS11 group permission [errno=%d].\n",
+				errno);
+			umask(proc_umask);
+			return rc;
+		}
+	}
+	umask(proc_umask);
+	return 0;
+}
+
 /*****************************************
  *  main() -
  *      You know what main does.
@@ -144,7 +298,7 @@ void run_sanity_checks()
  *****************************************/
 
 int main ( int argc, char *argv[], char *envp[]) {
-	int ret;
+	int ret, i;
 
 	/**********************************/
 	/* Read in command-line arguments */
@@ -228,6 +382,14 @@ int main ( int argc, char *argv[], char *envp[]) {
 		return 6;
 	}
 
+	/* Create customized token directories */
+	psinfo = &socketData.slot_info[0];
+	for (i = 0; i < NUMBER_SLOTS_MANAGED; i++, psinfo++) {
+		ret = chk_create_tokdir(psinfo);
+		if (ret)
+			return EACCES;
+	}
+
 	/*
 	 *  Become a Daemon, if called for
 	 */
